
//============================================
// SAWStudio FX API TestPlugIn - DLL Routines
//
// CopyRight 2001 By Bob Lentini
//============================================

#include <windows.h>
#include "FX_API_TestPlugIn.h"

//=========
// DllMain
//=========
BOOL    WINAPI      DllMain(HANDLE hDLL, DWORD dwReason, LPVOID lpReserved)

{
switch(dwReason)
    {
    case DLL_PROCESS_ATTACH:

        // Save Instance Handle For Later Use
        //-----------------------------------
        hInstFx = (HINSTANCE)hDLL;

        break;

    case DLL_THREAD_ATTACH:
        break;

    case DLL_THREAD_DETACH:
        break;

    case DLL_PROCESS_DETACH:
        break;
    }

return(TRUE);
}

//=======================
// SAWStudio API Routine
//
// FX Initialization
//=======================

void    __stdcall   FxInit(IQS_SAWFXDATA* SawFxDataPtr)

{
// Save Shared Memory Pointer
//---------------------------
FxPtrSawData = SawFxDataPtr;

// Take Data From Saw
//-------------------
FxPlugInIndex = FxPtrSawData->FX_PlugInIndex;
hWndSawMain = FxPtrSawData->FX_hWndMain;
hIQSSysFont = FxPtrSawData->FX_IQSSysFont;

FxProgramVersion = FxPtrSawData->FX_VersionNum;
FxMaxBufferSize = FxPtrSawData->FX_MaxBufferByteSize;

FxSampleRate = FxPtrSawData->FX_MTSampleRate;
FxResolution = FxPtrSawData->FX_MTResolution;

FxReturnTrkOffset = FxPtrSawData->FX_MultiTrackRtnTrackOffset;
FxOutputTrkOffset = FxPtrSawData->FX_MultiTrackOutTrackOffset;

// Initialize PlugIn
//------------------
InitializeProc();

// Reset All Parameter Sets
//-------------------------
ResetParamSet(FX_ALL_PARAM_SETS);

// Clear Process Variables
//------------------------
FxProcessParamSet = 0;
FxProcessFlag = PROCESS_IDLE;
FxPreScanFlag = PROCESS_IDLE;

// Send Back Title String
//-----------------------
lstrcpy(FxPtrSawData->FX_Title, szFxTitle);
return;
}

//=======================
// SAWStudio API Routine
//
// FX Main Handler
//=======================

void  __stdcall  FxMainHandler(DWORD TempFunctionID)

{
// Call The Proper Routine Based On The Function ID
//-------------------------------------------------
if(TempFunctionID == FX_SHUTDOWN)
    {
    PlugInShutDown();
    return;
    }

if(TempFunctionID == FX_BEGIN_PROCESS)
    {
    PlugInBegProcess();
    return;
    }

if(TempFunctionID == FX_END_PROCESS)
    {
    PlugInEndProcess();
    return;
    }

if(TempFunctionID == FX_BEGIN_XTRA)
    {
    return;
    }

if(TempFunctionID == FX_END_XTRA)
    {
    return;
    }

if(TempFunctionID == FX_OPEN_PLUGIN_WINDOW)
    {
    PlugInOpenWindow();
    return;
    }

if(TempFunctionID == FX_CLOSE_PLUGIN_WINDOW)
    {
    PlugInCloseWindow();
    return;
    }

if(TempFunctionID == FX_PATCH_NEW_PARAM_SET)
    {
    PlugInPatchNewParamSet();
    return;
    }

if(TempFunctionID == FX_RESET_PARAM_SET)
    {
    PlugInResetParamSet();
    return;
    }

if(TempFunctionID == FX_RECEIVE_PARAM_SET_DATA)
    {
    PlugInReceiveParamSetData();
    return;
    }

if(TempFunctionID == FX_SEND_PARAM_SET_DATA)
    {
    PlugInSendParamSetData();
    return;
    }

if(TempFunctionID == FX_INIT_PARAM_SET_DATA)
    {
    PlugInInitParamSetData();
    return;
    }

if(TempFunctionID == FX_CHANGE_MT_RATE)
    {
    PlugInChangeRate();
    return;
    }

if(TempFunctionID == FX_CHANGE_MT_RES)
    {
    PlugInChangeRes();
    return;
    }

if(TempFunctionID == FX_CHANGE_MT_ACTIVE_TRACK)
    {
    PlugInChangeActiveTrack();
    return;
    }

if(TempFunctionID == FX_CHANGE_MT_MARKS)
    {
    PlugInChangeMTMarks();
    return;
    }

if(TempFunctionID == FX_CHANGE_SMPTE_INFO)
    {
    PlugInChangeSmpteInfo();
    return;
    }

if(TempFunctionID == FX_CHANGE_TEMPO_INFO)
    {
    PlugInChangeTempoInfo();
    return;
    }

if(TempFunctionID == FX_CLEAR_CHASE_AUTOMATION)
    {
    PlugInClearChaseAutomation();
    return;
    }

if(TempFunctionID == FX_STORE_CHASE_AUTOMATION)
    {
    PlugInStoreChaseAutomation();
    return;
    }

if(TempFunctionID == FX_DISPLAY_CHASE_AUTOMATION)
    {
    PlugInDisplayChaseAutomation();
    return;
    }

if(TempFunctionID == FX_RESYNC_CHASE_AUTOMATION)
    {
    PlugInReSyncChaseAutomation();
    return;
    }

if(TempFunctionID == FX_CHANGE_PROCESS_AUTOMATION)
    {
    PlugInChangeProcessAutomation();
    return;
    }

if(TempFunctionID == FX_DISPLAY_PROCESS_AUTOMATION)
    {
    PlugInDisplayProcessAutomation();
    return;
    }

return;
}

//=======================
// SAWStudio API Routine
//
// FX Process Buffer
//=======================

void  __stdcall  FxProcessBuffer(DWORD ParamSet, LPBYTE BuffPtr)

{
// Set Process Variables For This Call
//------------------------------------
FxProcessParamSet = ParamSet;
FxProcessBuffPtr = BuffPtr;
FxProcessBuffSize = FxPtrSawData->FX_BufferByteSize;
FxProcessBytesPerSample = FxPtrSawData->FX_BufferBytesPerSample;
FxProcessPos = FxPtrSawData->FX_CurProcessPos;

// This Plug-In Only Processes Stereo Buffers
//-------------------------------------------
if(FxPtrSawData->FX_BufferChans != 2)
    return;

// Skip Process If PlugIn Is Bypassed
//-----------------------------------
if(PlugInDataValue[FxProcessParamSet][AUTO_BYPASS_SW])
    return;

// Process The Correct Buffer Format
//----------------------------------
if(FxProcessBytesPerSample == 4)
    DoPlugInProcess32();
else
    DoPlugInProcess16();

// Set Final BufferSize If Your Process Changes It
// Note That The Size Can Not Grow Beyond FxMaxBufferSize
//-------------------------------------------------------
FxPtrSawData->FX_BufferByteSize = FxProcessBuffSize;

return;
}

//=======================
// SAWStudio API Routine
//
// FX Change Position
//=======================

void  __stdcall  FxChangePosition(void)

{
// Get Parameter Data
//-------------------
FxSamplePos = FxPtrSawData->FX_CurSamplePos;

// Display All Meters At Current Sample Position Reference
//--------------------------------------------------------
DisplayAllMeters();
return;
}

//===========================
// SAWStudio Handler Routine
//
// PlugIn Shutdown
//===========================

void    __stdcall   PlugInShutDown(void)

{
// ShutDown PlugIn
//----------------
ShutdownProc();

return;
}

//===========================
// SAWStudio Handler Routine
//
// PlugIn Begin Process
//===========================

void    __stdcall   PlugInBegProcess(void)

{
// Set Process Variables For This Call
//------------------------------------
FxProcessParamSet = FxPtrSawData->FX_Handler_DWord_Param1;
FxProcessBytesPerSample = FxPtrSawData->FX_Handler_DWord_Param2;
FxProcessFlag = FxPtrSawData->FX_ProcessFlag;

if(FxProcessFlag == DATA_PRESCAN)
    FxPreScanFlag = 1;
else
    FxPreScanFlag = 0;

// Init This Parameter Set For Processing
//---------------------------------------
InitPlugInVariables(FxProcessParamSet, FX_ALL_CONTROLS);
InitPlugInMeter(FxProcessParamSet);

return;
}

//===========================
// SAWStudio Handler Routine
//
// PlugIn End Process
//===========================

void    __stdcall   PlugInEndProcess(void)

{
// Reset Process Variables
//------------------------
FxProcessParamSet = FxPtrSawData->FX_Handler_DWord_Param1;

FxProcessFlag = PROCESS_IDLE;
FxPreScanFlag = PROCESS_IDLE;

// CleanUp Parameter Set After Processing If Needed
//-------------------------------------------------
ResetMeterDisplay(FxProcessParamSet);

return;
}

//===========================
// SAWStudio Handler Routine
//
// PlugIn Open Window
//===========================

void    __stdcall   PlugInOpenWindow(void)

{
DWORD   TempParamSet;
HWND    TempRefWindow;

// Get Parameter Data
//-------------------
TempParamSet = FxPtrSawData->FX_Handler_DWord_Param1;
TempRefWindow = (HWND)FxPtrSawData->FX_Handler_DWord_Param3;

if(PlugInParamSetTable[TempParamSet] == 0)
    {
    // Missing Patch... Add It
    //------------------------
    MessageBox(NULL, CorruptedErrorMsg, WarningMsg, MB_TASKMODAL | MB_TOPMOST | MB_ICONEXCLAMATION | MB_OK);

    // Save Track Information
    //-----------------------
    PlugInTrack[TempParamSet] = FxPtrSawData->FX_Handler_DWord_Param2;

    // Save Format Information
    //------------------------
    PlugInBytesPerSample[TempParamSet] = FxPtrSawData->FX_Handler_DWord_Param4;

    // Init PlugIn Variables If Needed
    //--------------------------------
    InitPlugInVariables(TempParamSet, FX_ALL_CONTROLS);

    // Mark This Set Active In Table
    //------------------------------
    PlugInParamSetTable[TempParamSet] = 1;
            
    // Increment Active Set Count
    //---------------------------
    PlugInParamSetCount++;
    }

// Open PlugIn Window
//-------------------
CreatePlugInWindow(TempParamSet, TempRefWindow);
return;
}

//===========================
// SAWStudio Handler Routine
//
// PlugIn Close Window
//===========================

void    __stdcall   PlugInCloseWindow(void)

{
DWORD   TempParamSet;

// Get Parameter Data
//-------------------
TempParamSet = FxPtrSawData->FX_Handler_DWord_Param1;

// Close PlugIn Window
//--------------------
DestroyPlugInWindow(TempParamSet);
return;
}

//================================
// SAWStudio Handler Routine
//
// PlugIn Patch New Parameter Set
//================================

void    __stdcall   PlugInPatchNewParamSet(void)

{
DWORD   TempParamSet;

// Are There Any Available Parameter Sets?
//----------------------------------------
if(PlugInParamSetCount < MAX_PARAM_SETS)
    {
    // Search For The First Available Set
    //-----------------------------------
    for(TempParamSet = 0; TempParamSet < MAX_PARAM_SETS; TempParamSet++)
        {
        if(PlugInParamSetTable[TempParamSet] == 0)
            {
            // Save Format Information
            //------------------------
            PlugInBytesPerSample[TempParamSet] = FxPtrSawData->FX_Handler_DWord_Param1;

            // Save Track Information
            //-----------------------
            PlugInTrack[TempParamSet] = FxPtrSawData->FX_Handler_DWord_Param2;

            // Init PlugIn Variables If Needed
            //--------------------------------
            InitPlugInVariables(TempParamSet, FX_ALL_CONTROLS);

            // Pass New Parameter Set Value Back To Saw
            //-----------------------------------------
            FxPtrSawData->FX_ParamSet = TempParamSet;

            // Mark This Set Active In Table
            //------------------------------
            PlugInParamSetTable[TempParamSet] = 1;
            
            // Increment Active Set Count
            //---------------------------
            PlugInParamSetCount++;
            return;
            }
        }
    }

// None Available
//---------------
MessageBox(NULL, MaximumAssignmentMsg, WarningMsg, MB_TASKMODAL | MB_TOPMOST | MB_ICONEXCLAMATION | MB_OK);

FxPtrSawData->FX_ParamSet = NONE_AVAILABLE;
return;
}

//============================
// SAWStudio Handler Routine
//
// PlugIn Reset Parameter Set
//============================

void    __stdcall   PlugInResetParamSet(void)

{
DWORD   TempParamSet;

// Get Parameter Data
//-------------------
TempParamSet = FxPtrSawData->FX_Handler_DWord_Param1;

// Reset Parameter Set
//--------------------
ResetParamSet(TempParamSet);

// If The Window Exists... Close It
//---------------------------------
if(hWndPlugIn[TempParamSet])
    DestroyPlugInWindow(TempParamSet);

return;
}

//===================================
// SAWStudio Handler Routine
//
// PlugIn Receive Parameter Set Data
//===================================

void    __stdcall   PlugInReceiveParamSetData(void)

{
DWORD   TempParamSet;
DWORD   TempBuffSize;
LPBYTE  TempBuffPtr;
DWORD   TempOffset;
DWORD   TempCount;
DWORD   TempMaxCount;
DWORD   TempSet;
DWORD   TempVisibleFlag;

// Get Parameter Data
//-------------------
TempParamSet = FxPtrSawData->FX_Handler_DWord_Param1;
TempBuffSize = FxPtrSawData->FX_Handler_DWord_Param2;
TempBuffPtr = FxPtrSawData->FX_Handler_Ptr_Param1;

// Check Header For A Match
//-------------------------
if(lstrcmp((LPSTR)TempBuffPtr, "FX API TEST PLUGIN PRESETS 001 ") != 0)
    {
    // Not Proper Format
    //------------------
    FxPtrSawData->FX_Handler_DWord_Param3 = 1;
    return;
    }

// Offset Past The Header
//-----------------------
TempOffset = 32;

// Reset The Parameter Set(s) (Might Be All Sets)
//-----------------------------------------------
ResetParamSet(TempParamSet);

// Set Max Count
//--------------
if(TempParamSet == FX_ALL_PARAM_SETS)
    TempMaxCount = MAX_PARAM_SETS;
else
    TempMaxCount = 1;

// Read Individual Settings For Each Active ParamSet
//--------------------------------------------------
for(TempCount = 0; TempCount <= TempMaxCount; TempCount++)
    {
    // Get Parameter Set Value
    //------------------------
    TempSet = *(PDWORD)(TempBuffPtr + TempOffset);
    TempOffset = TempOffset + 4;

    // End Of List... Exit Loop
    //-------------------------
    if(TempSet == END_OF_LIST)
        break;

    // Read PlugIn Data For This Set
    //------------------------------
    PlugInTrack[TempSet] = *(PDWORD)(TempBuffPtr + TempOffset);
    TempOffset = TempOffset + 4;

    PlugInBytesPerSample[TempSet] = *(PDWORD)(TempBuffPtr + TempOffset);
    TempOffset = TempOffset + 4;

    PlugInDefaultDataValue[TempSet][AUTO_BYPASS_SW] = *(PDWORD)(TempBuffPtr + TempOffset);
    TempOffset = TempOffset + 4;

    PlugInDefaultDataValue[TempSet][AUTO_VOL_POT] = *(PDWORD)(TempBuffPtr + TempOffset);
    TempOffset = TempOffset + 4;

    PlugInDefaultDataValue[TempSet][AUTO_VALUE_1] = *(PDWORD)(TempBuffPtr + TempOffset);
    TempOffset = TempOffset + 4;

    PlugInDefaultDataValue[TempSet][AUTO_VALUE_2] = *(PDWORD)(TempBuffPtr + TempOffset);
    TempOffset = TempOffset + 4;

    PlugInDefaultDataValue[TempSet][AUTO_VALUE_3] = *(PDWORD)(TempBuffPtr + TempOffset);
    TempOffset = TempOffset + 4;

    PlugInDefaultDataValue[TempSet][AUTO_VALUE_4] = *(PDWORD)(TempBuffPtr + TempOffset);
    TempOffset = TempOffset + 4;

    PlugInDefaultDataValue[TempSet][AUTO_VALUE_5] = *(PDWORD)(TempBuffPtr + TempOffset);
    TempOffset = TempOffset + 4;

    PlugInDefaultDataValue[TempSet][AUTO_VALUE_6] = *(PDWORD)(TempBuffPtr + TempOffset);
    TempOffset = TempOffset + 4;

    // Set All Current Values To New Defaults
    //---------------------------------------
    CopyMemory(&PlugInDataValue[TempSet][0], &PlugInDefaultDataValue[TempSet][0], MAX_AUTO_CONTROLS * 4);

    // Init PlugIn Variables
    //----------------------
    InitPlugInVariables(TempSet, FX_ALL_CONTROLS);

    // Mark This Set Active In Table
    //------------------------------
    PlugInParamSetTable[TempSet] = 1;

    // Increment Active Set Count
    //---------------------------
    PlugInParamSetCount++;

    // Read Window Info
    //-----------------
    PlugInXStart[TempSet] = *(PDWORD)(TempBuffPtr + TempOffset);
    TempOffset = TempOffset + 4;

    PlugInYStart[TempSet] = *(PDWORD)(TempBuffPtr + TempOffset);
    TempOffset = TempOffset + 4;

    PlugInWidth[TempSet] = *(PDWORD)(TempBuffPtr + TempOffset);
    TempOffset = TempOffset + 4;

    PlugInHeight[TempSet] = *(PDWORD)(TempBuffPtr + TempOffset);
    TempOffset = TempOffset + 4;

    TempVisibleFlag = *(PDWORD)(TempBuffPtr + TempOffset);
    TempOffset = TempOffset + 4;

    // Display Window If Needed
    //-------------------------
    if(TempVisibleFlag)
        CreatePlugInWindow(TempSet, NULL);
    }

// Everything OK
//--------------
FxPtrSawData->FX_Handler_DWord_Param3 = 0;

return;
}

//================================
// SAWStudio Handler Routine
//
// PlugIn Send Parameter Set Data
//================================

void    __stdcall   PlugInSendParamSetData(void)

{
DWORD   TempParamSet;
DWORD   TempBuffSize;
LPBYTE  TempBuffPtr;
DWORD   TempSet;
DWORD   TempMaxCount;
RECT    rWindow;

// Get Parameter Data
//-------------------
TempParamSet = FxPtrSawData->FX_Handler_DWord_Param1;
TempBuffPtr = FxPtrSawData->FX_Handler_Ptr_Param1;

// If No Active Parameter Sets... Return Zero Buffer Size
//-------------------------------------------------------
if(PlugInParamSetCount == 0)
    {
    FxPtrSawData->FX_Handler_DWord_Param2 = 0;
    return;
    }

// Set Max Count (Might Be All Sets)
//----------------------------------
if(TempParamSet == FX_ALL_PARAM_SETS)
    TempMaxCount = MAX_PARAM_SETS;
else
    TempMaxCount = 1;

lstrcpy((LPSTR)TempBuffPtr, "FX API TEST PLUGIN PRESETS 001 ");
TempBuffSize = 32;

// Save Individual Settings For Each Active ParamSet
//--------------------------------------------------
for(TempSet = 0; TempSet < TempMaxCount; TempSet++)
    {
    // Skip If Not Active
    //-------------------
    if(PlugInParamSetTable[TempSet] == 0)
        continue;

    // Write Parameter Set Value
    //--------------------------
    *(PDWORD)(TempBuffPtr + TempBuffSize) = TempSet;
    TempBuffSize = TempBuffSize + 4;

    // Write PlugIn Data
    //------------------
    *(PDWORD)(TempBuffPtr + TempBuffSize) = PlugInTrack[TempSet];
    TempBuffSize = TempBuffSize + 4;

    *(PDWORD)(TempBuffPtr + TempBuffSize) = PlugInBytesPerSample[TempSet];
    TempBuffSize = TempBuffSize + 4;

    *(PDWORD)(TempBuffPtr + TempBuffSize) = PlugInDefaultDataValue[TempSet][AUTO_BYPASS_SW];
    TempBuffSize = TempBuffSize + 4;

    *(PDWORD)(TempBuffPtr + TempBuffSize) = PlugInDefaultDataValue[TempSet][AUTO_VOL_POT];
    TempBuffSize = TempBuffSize + 4;

    *(PDWORD)(TempBuffPtr + TempBuffSize) = PlugInDefaultDataValue[TempSet][AUTO_VALUE_1];
    TempBuffSize = TempBuffSize + 4;

    *(PDWORD)(TempBuffPtr + TempBuffSize) = PlugInDefaultDataValue[TempSet][AUTO_VALUE_2];
    TempBuffSize = TempBuffSize + 4;

    *(PDWORD)(TempBuffPtr + TempBuffSize) = PlugInDefaultDataValue[TempSet][AUTO_VALUE_3];
    TempBuffSize = TempBuffSize + 4;

    *(PDWORD)(TempBuffPtr + TempBuffSize) = PlugInDefaultDataValue[TempSet][AUTO_VALUE_4];
    TempBuffSize = TempBuffSize + 4;

    *(PDWORD)(TempBuffPtr + TempBuffSize) = PlugInDefaultDataValue[TempSet][AUTO_VALUE_5];
    TempBuffSize = TempBuffSize + 4;

    *(PDWORD)(TempBuffPtr + TempBuffSize) = PlugInDefaultDataValue[TempSet][AUTO_VALUE_6];
    TempBuffSize = TempBuffSize + 4;

    // Write Window Info
    //------------------
    if((hWndPlugIn[TempSet]) == 0)
        {
        // Not Visible
        //------------
        *(PDWORD)(TempBuffPtr + TempBuffSize) = 0;
        TempBuffSize = TempBuffSize + 4;

        *(PDWORD)(TempBuffPtr + TempBuffSize) = 0;
        TempBuffSize = TempBuffSize + 4;

        *(PDWORD)(TempBuffPtr + TempBuffSize) = 0;
        TempBuffSize = TempBuffSize + 4;

        *(PDWORD)(TempBuffPtr + TempBuffSize) = 0;
        TempBuffSize = TempBuffSize + 4;

        *(PDWORD)(TempBuffPtr + TempBuffSize) = 0;
        TempBuffSize = TempBuffSize + 4;
        }
    else
        {
        // Visible
        //--------
        GetWindowRect(hWndPlugIn[TempSet], &rWindow);
    
        *(PDWORD)(TempBuffPtr + TempBuffSize) = rWindow.left;
        TempBuffSize = TempBuffSize + 4;
    
        *(PDWORD)(TempBuffPtr + TempBuffSize) = rWindow.top;
        TempBuffSize = TempBuffSize + 4;

        *(PDWORD)(TempBuffPtr + TempBuffSize) = rWindow.right - rWindow.left;
        TempBuffSize = TempBuffSize + 4;

        *(PDWORD)(TempBuffPtr + TempBuffSize) = rWindow.bottom - rWindow.top;
        TempBuffSize = TempBuffSize + 4;

        *(PDWORD)(TempBuffPtr + TempBuffSize) = 1;
        TempBuffSize = TempBuffSize + 4;
        }        
    }

// Write End Of List
//------------------
*(PDWORD)(TempBuffPtr + TempBuffSize) = END_OF_LIST;
TempBuffSize = TempBuffSize + 4;

// Return Buffer Size
//-------------------
FxPtrSawData->FX_Handler_DWord_Param2 = TempBuffSize;

return;
}

//================================
// SAWStudio Handler Routine
//
// PlugIn Init Parameter Set Data
//================================

void    __stdcall   PlugInInitParamSetData(void)

{
DWORD   TempParamSet;

// Get Parameter Data
//-------------------
TempParamSet = FxPtrSawData->FX_Handler_DWord_Param1;

// Init PlugIn Variables
//----------------------
InitPlugInVariables(TempParamSet, FX_ALL_CONTROLS);

return;
}

//===========================
// SAWStudio Handler Routine
//
// PlugIn Change Sample Rate
//===========================

void    __stdcall   PlugInChangeRate(void)

{
DWORD   TempParamSet;
DWORD   TempNewRate;

// Get Parameter Data
//-------------------
TempNewRate = FxPtrSawData->FX_Handler_DWord_Param1;

// Init New Data If Rate Is Different
//-----------------------------------
if(FxSampleRate != TempNewRate)
    {
    FxSampleRate = TempNewRate;
    
    for(TempParamSet = 0; TempParamSet < MAX_PARAM_SETS; TempParamSet++)
        {
        if(PlugInParamSetTable[TempParamSet] == 0)
            continue;

        // Init PlugIn Variables
        //----------------------
        InitPlugInVariables(TempParamSet, FX_ALL_CONTROLS);
        }
    }

return;
}

//===========================
// SAWStudio Handler Routine
//
// PlugIn Change Resolution
//===========================

void    __stdcall   PlugInChangeRes(void)

{
DWORD   TempParamSet;
DWORD   TempNewRes;

// Get Parameter Data
//-------------------
TempNewRes = FxPtrSawData->FX_Handler_DWord_Param1;

// Init New Data If Res Is Different
//----------------------------------
if(FxResolution != TempNewRes)
    {
    FxResolution = TempNewRes;

    for(TempParamSet = 0; TempParamSet < MAX_PARAM_SETS; TempParamSet++)
        {
        if(PlugInParamSetTable[TempParamSet] == 0)
            continue;

        // Init PlugIn Variables
        //----------------------
        InitPlugInVariables(TempParamSet, FX_ALL_CONTROLS);
        }
    }

return;
}

//============================
// SAWStudio Handler Routine
//
// PlugIn Change Active Track
//============================

void    __stdcall   PlugInChangeActiveTrack(void)

{
DWORD   TempNewTrack;

// Get Parameter Data
//-------------------
TempNewTrack = FxPtrSawData->FX_Handler_DWord_Param1;

// Perform Any Track Change Related Process Needed
//------------------------------------------------

return;
}

//===========================
// SAWStudio Handler Routine
//
// PlugIn Change MT Marks
//===========================

void    __stdcall   PlugInChangeMTMarks(void)

{
DWORD   TempNewMarkBegPos;
DWORD   TempNewMarkEndPos;

// Get Parameter Data
//-------------------
TempNewMarkBegPos = FxPtrSawData->FX_Handler_DWord_Param1;
TempNewMarkEndPos = FxPtrSawData->FX_Handler_DWord_Param2;

// Perform Any Marked Area Change Related Process Needed
//------------------------------------------------------

return;
}

//===========================
// SAWStudio Handler Routine
//
// PlugIn Change Smpte Info
//===========================

void    __stdcall   PlugInChangeSmpteInfo(void)

{
// The Smpte Offset Shared Memory Variables Are Set
// The Smpte Format Shared Memory Variable Is Set
// The Smpte Mode Shared Memory Variable Is Set
//
// Perform Any Smpte Info Change Related Process Needed
//-----------------------------------------------------

return;
}

//===========================
// SAWStudio Handler Routine
//
// PlugIn Change Tempo Info
//===========================

void    __stdcall   PlugInChangeTempoInfo(void)

{
// The Tempo Shared Memory Variables Are Set
//
// Perform Any Tempo Info Change Related Process Needed
//-----------------------------------------------------

return;
}

//===========================
// SAWStudio Handler Routine
//
// Clear Chase Automation
//===========================

void    __stdcall   PlugInClearChaseAutomation(void)

{
DWORD   TempParamSet;
DWORD   TempPhysicalTrack;
DWORD   TempSet;
DWORD   TempStartSet;
DWORD   TempEndSet;

// No Active Parameter Sets... Nothing To Do
//------------------------------------------
if(PlugInParamSetCount == 0)
    return;

// Get Parameter Data
//-------------------
TempParamSet = FxPtrSawData->FX_Handler_DWord_Param1;
TempPhysicalTrack = FxPtrSawData->FX_Handler_DWord_Param2;

// Set Start And End Loop Values (Might Be All Sets)
//--------------------------------------------------
if(TempParamSet == FX_ALL_PARAM_SETS)
    {
    TempStartSet = 0;
    TempEndSet = MAX_PARAM_SETS;
    }
else
    {
    // Verify A Track Match For This Parameter Set
    //--------------------------------------------
    if(TempPhysicalTrack != PlugInTrack[TempParamSet])
        return;

    TempStartSet = TempParamSet;
    TempEndSet = TempParamSet + 1;
    }

for(TempSet = TempStartSet; TempSet < TempEndSet; TempSet++)
    {
    // Skip All Inactive Parameter Sets
    //---------------------------------
    if(PlugInParamSetTable[TempSet] == 0)
        continue;

    // Copy Current Data Values To Automation Data Storage
    //----------------------------------------------------
    CopyMemory(&PlugInAutoDataValue[TempSet][0], &PlugInDataValue[TempSet][0], MAX_AUTO_CONTROLS * 4);

    // Set Current Data Values To Default Values
    //------------------------------------------
    CopyMemory(&PlugInDataValue[TempSet][0], &PlugInDefaultDataValue[TempSet][0], MAX_AUTO_CONTROLS * 4);
    }

return;
}

//===========================
// SAWStudio Handler Routine
//
// Store Chase Automation
//===========================

void    __stdcall   PlugInStoreChaseAutomation(void)

{
DWORD   TempParamSet;
DWORD   TempPhysicalTrack;
DWORD   TempCtrlId;
int     TempDataValue;

// Get Parameter Data
//-------------------
TempParamSet = FxPtrSawData->FX_Handler_DWord_Param1;

// Skip If Inactive
//-----------------
if(PlugInParamSetTable[TempParamSet] == 0)
    return;

TempPhysicalTrack = FxPtrSawData->FX_Handler_DWord_Param2;

// Verify A Track Match For This Parameter Set
//--------------------------------------------
if(TempPhysicalTrack != PlugInTrack[TempParamSet])
    return;

// Set New Data Value
//-------------------
TempCtrlId = FxPtrSawData->FX_Handler_DWord_Param3;
TempDataValue = FxPtrSawData->FX_Handler_Int_Param1;

PlugInDataValue[TempParamSet][TempCtrlId] = TempDataValue;
return;
}

//===========================
// SAWStudio Handler Routine
//
// Display Chase Automation
//===========================

void    __stdcall   PlugInDisplayChaseAutomation(void)

{
DWORD   TempParamSet;
DWORD   TempPhysicalTrack;
DWORD   TempSet;
DWORD   TempStartSet;
DWORD   TempEndSet;

// No Active Parameter Sets... Nothing To Display
//-----------------------------------------------
if(PlugInParamSetCount == 0)
    return;

// Get Parameter Data
//-------------------
TempParamSet = FxPtrSawData->FX_Handler_DWord_Param1;
TempPhysicalTrack = FxPtrSawData->FX_Handler_DWord_Param2;

// Set Start And End Loop Values (Might Be All Sets)
//--------------------------------------------------
if(TempParamSet == FX_ALL_PARAM_SETS)
    {
    TempStartSet = 0;
    TempEndSet = MAX_PARAM_SETS;
    }
else
    {
    // Verify A Track Match For This Parameter Set
    //--------------------------------------------
    if(TempPhysicalTrack != PlugInTrack[TempParamSet])
        return;

    TempStartSet = TempParamSet;
    TempEndSet = TempParamSet + 1;
    }

for(TempSet = TempStartSet; TempSet < TempEndSet; TempSet++)
    {
    // Skip If Window Is Not Open
    //---------------------------
    if(hWndPlugIn[TempSet] == 0)
        continue;

    // Display All Changed Values
    //---------------------------
    if(PlugInDataValue[TempSet][AUTO_BYPASS_SW] != PlugInAutoDataValue[TempSet][AUTO_BYPASS_SW])
        {
        if(PlugInDataValue[TempSet][AUTO_BYPASS_SW])
            DisplayBitmapBtn(PlugInDC[TempSet], &RectPlugInBypassDisplayBtn, hPlugInBypassBtnBitmap);
        else
            RefreshBitmap(PlugInDC[TempSet], &RectPlugInBypassDisplayBtn, hPlugInMainBitmap);
        }

    if(PlugInDataValue[TempSet][AUTO_VOL_POT] != PlugInAutoDataValue[TempSet][AUTO_VOL_POT])
        DrawVertKnob(PlugInDC[TempSet], &RectPlugInVolPot, PlugInDataValue[TempSet][AUTO_VOL_POT], 1);

    if(PlugInDataValue[TempSet][AUTO_VALUE_1] != PlugInAutoDataValue[TempSet][AUTO_VALUE_1])
        DisplayPlugInValue(PlugInDC[TempSet], AUTO_VALUE_1, PlugInDataValue[TempSet][AUTO_VALUE_1], 1);

    if(PlugInDataValue[TempSet][AUTO_VALUE_2] != PlugInAutoDataValue[TempSet][AUTO_VALUE_2])
        DisplayPlugInValue(PlugInDC[TempSet], AUTO_VALUE_2, PlugInDataValue[TempSet][AUTO_VALUE_2], 1);

    if(PlugInDataValue[TempSet][AUTO_VALUE_3] != PlugInAutoDataValue[TempSet][AUTO_VALUE_3])
        DisplayPlugInValue(PlugInDC[TempSet], AUTO_VALUE_3, PlugInDataValue[TempSet][AUTO_VALUE_3], 1);

    if(PlugInDataValue[TempSet][AUTO_VALUE_4] != PlugInAutoDataValue[TempSet][AUTO_VALUE_4])
        DisplayPlugInValue(PlugInDC[TempSet], AUTO_VALUE_4, PlugInDataValue[TempSet][AUTO_VALUE_4], 1);

    if(PlugInDataValue[TempSet][AUTO_VALUE_5] != PlugInAutoDataValue[TempSet][AUTO_VALUE_5])
        DisplayPlugInValue(PlugInDC[TempSet], AUTO_VALUE_5, PlugInDataValue[TempSet][AUTO_VALUE_5], 1);

    if(PlugInDataValue[TempSet][AUTO_VALUE_6] != PlugInAutoDataValue[TempSet][AUTO_VALUE_6])
        DisplayPlugInValue(PlugInDC[TempSet], AUTO_VALUE_6, PlugInDataValue[TempSet][AUTO_VALUE_6], 1);
    }

return;
}

//===========================
// SAWStudio Handler Routine
//
// ReSync Chase Automation
//===========================

void    __stdcall   PlugInReSyncChaseAutomation(void)

{
DWORD   TempParamSet;
DWORD   TempPhysicalTrack;

// Get Parameter Data
//-------------------
TempParamSet = FxPtrSawData->FX_Handler_DWord_Param1;

// Skip If Inactive
//-----------------
if(PlugInParamSetTable[TempParamSet] == 0)
    return;

TempPhysicalTrack = FxPtrSawData->FX_Handler_DWord_Param2;

// Verify A Track Match For This Parameter Set
//--------------------------------------------
if(TempPhysicalTrack != PlugInTrack[TempParamSet])
    return;

// Initialize And Display All Changed Values
//------------------------------------------
if(PlugInDataValue[TempParamSet][AUTO_BYPASS_SW] != PlugInAutoDataValue[TempParamSet][AUTO_BYPASS_SW])
    {
    InitPlugInVariables(TempParamSet, AUTO_BYPASS_SW);

    // Display If Window Is Open
    //--------------------------
    if(hWndPlugIn[TempParamSet])
        {
        if(PlugInDataValue[TempParamSet][AUTO_BYPASS_SW])
            DisplayBitmapBtn(PlugInDC[TempParamSet], &RectPlugInBypassDisplayBtn, hPlugInBypassBtnBitmap);

        else
            RefreshBitmap(PlugInDC[TempParamSet], &RectPlugInBypassDisplayBtn, hPlugInMainBitmap);
        }
    }

if(PlugInDataValue[TempParamSet][AUTO_VOL_POT] != PlugInAutoDataValue[TempParamSet][AUTO_VOL_POT])
    {
    InitPlugInVariables(TempParamSet, AUTO_VOL_POT);

    // Display If Window Is Open
    //--------------------------
    if(hWndPlugIn[TempParamSet])
        DrawVertKnob(PlugInDC[TempParamSet], &RectPlugInVolPot, PlugInDataValue[TempParamSet][AUTO_VOL_POT], 1);
    }

if(PlugInDataValue[TempParamSet][AUTO_VALUE_1] != PlugInAutoDataValue[TempParamSet][AUTO_VALUE_1])
    {
    InitPlugInVariables(TempParamSet, AUTO_VALUE_1);

    // Display If Window Is Open
    //--------------------------
    if(hWndPlugIn[TempParamSet])
        DisplayPlugInValue(PlugInDC[TempParamSet], AUTO_VALUE_1, PlugInDataValue[TempParamSet][AUTO_VALUE_1], 1);
    }

if(PlugInDataValue[TempParamSet][AUTO_VALUE_2] != PlugInAutoDataValue[TempParamSet][AUTO_VALUE_2])
    {
    InitPlugInVariables(TempParamSet, AUTO_VALUE_2);

    // Display If Window Is Open
    //--------------------------
    if(hWndPlugIn[TempParamSet])
        DisplayPlugInValue(PlugInDC[TempParamSet], AUTO_VALUE_2, PlugInDataValue[TempParamSet][AUTO_VALUE_2], 1);
    }

if(PlugInDataValue[TempParamSet][AUTO_VALUE_3] != PlugInAutoDataValue[TempParamSet][AUTO_VALUE_3])
    {
    InitPlugInVariables(TempParamSet, AUTO_VALUE_3);

    // Display If Window Is Open
    //--------------------------
    if(hWndPlugIn[TempParamSet])
        DisplayPlugInValue(PlugInDC[TempParamSet], AUTO_VALUE_3, PlugInDataValue[TempParamSet][AUTO_VALUE_3], 1);
    }

if(PlugInDataValue[TempParamSet][AUTO_VALUE_4] != PlugInAutoDataValue[TempParamSet][AUTO_VALUE_4])
    {
    InitPlugInVariables(TempParamSet, AUTO_VALUE_4);

    // Display If Window Is Open
    //--------------------------
    if(hWndPlugIn[TempParamSet])
        DisplayPlugInValue(PlugInDC[TempParamSet], AUTO_VALUE_4, PlugInDataValue[TempParamSet][AUTO_VALUE_4], 1);
    }

if(PlugInDataValue[TempParamSet][AUTO_VALUE_5] != PlugInAutoDataValue[TempParamSet][AUTO_VALUE_5])
    {
    InitPlugInVariables(TempParamSet, AUTO_VALUE_5);

    // Display If Window Is Open
    //--------------------------
    if(hWndPlugIn[TempParamSet])
        DisplayPlugInValue(PlugInDC[TempParamSet], AUTO_VALUE_5, PlugInDataValue[TempParamSet][AUTO_VALUE_5], 1);
    }

if(PlugInDataValue[TempParamSet][AUTO_VALUE_6] != PlugInAutoDataValue[TempParamSet][AUTO_VALUE_6])
    {
    InitPlugInVariables(TempParamSet, AUTO_VALUE_6);

    // Display If Window Is Open
    //--------------------------
    if(hWndPlugIn[TempParamSet])
        DisplayPlugInValue(PlugInDC[TempParamSet], AUTO_VALUE_6, PlugInDataValue[TempParamSet][AUTO_VALUE_6], 1);
    }

return;
}

//===========================
// SAWStudio Handler Routine
//
// Change Process Automation
//===========================

void    __stdcall   PlugInChangeProcessAutomation(void)

{
DWORD   TempParamSet;
DWORD   TempPhysicalTrack;
DWORD   TempCtrlId;
int     TempDataValue;

// Get Parameter Data
//-------------------
TempParamSet = FxPtrSawData->FX_Handler_DWord_Param1;

// Skip If Inactive
//-----------------
if(PlugInParamSetTable[TempParamSet] == 0)
    return;

TempPhysicalTrack = FxPtrSawData->FX_Handler_DWord_Param2;

// Verify A Track Match For This Parameter Set
//--------------------------------------------
if(TempPhysicalTrack != PlugInTrack[TempParamSet])
    return;

// Set New Data Value And Initialize It
//-------------------------------------
TempCtrlId = FxPtrSawData->FX_Handler_DWord_Param3;
TempDataValue = FxPtrSawData->FX_Handler_Int_Param1;

if(TempCtrlId == AUTO_BYPASS_SW)
    {
    PlugInDataValue[TempParamSet][AUTO_BYPASS_SW] = TempDataValue;

    InitPlugInVariables(TempParamSet, AUTO_BYPASS_SW);
    return;
    }

if(TempCtrlId == AUTO_VOL_POT)
    {
    PlugInDataValue[TempParamSet][AUTO_VOL_POT] = TempDataValue;

    InitPlugInVariables(TempParamSet, AUTO_VOL_POT);
    return;
    }

if(TempCtrlId == AUTO_VALUE_1)
    {
    PlugInDataValue[TempParamSet][AUTO_VALUE_1] = TempDataValue;

    InitPlugInVariables(TempParamSet, AUTO_VALUE_1);
    return;
    }

if(TempCtrlId == AUTO_VALUE_2)
    {
    PlugInDataValue[TempParamSet][AUTO_VALUE_2] = TempDataValue;

    InitPlugInVariables(TempParamSet, AUTO_VALUE_2);
    return;
    }

if(TempCtrlId == AUTO_VALUE_3)
    {
    PlugInDataValue[TempParamSet][AUTO_VALUE_3] = TempDataValue;

    InitPlugInVariables(TempParamSet, AUTO_VALUE_3);
    return;
    }

if(TempCtrlId == AUTO_VALUE_4)
    {
    PlugInDataValue[TempParamSet][AUTO_VALUE_4] = TempDataValue;

    InitPlugInVariables(TempParamSet, AUTO_VALUE_4);
    return;
    }

if(TempCtrlId == AUTO_VALUE_5)
    {
    PlugInDataValue[TempParamSet][AUTO_VALUE_5] = TempDataValue;

    InitPlugInVariables(TempParamSet, AUTO_VALUE_5);
    return;
    }

if(TempCtrlId == AUTO_VALUE_6)
    {
    PlugInDataValue[TempParamSet][AUTO_VALUE_6] = TempDataValue;

    InitPlugInVariables(TempParamSet, AUTO_VALUE_6);
    return;
    }

return;
}

//============================
// SAWStudio Handler Routine
//
// Display Process Automation
//============================

void    __stdcall   PlugInDisplayProcessAutomation(void)

{
DWORD   TempParamSet;
DWORD   TempPhysicalTrack;
DWORD   TempCtrlId;
int     TempDataValue;

// Get Parameter Data
//-------------------
TempParamSet = FxPtrSawData->FX_Handler_DWord_Param1;

// Skip If Inactive
//-----------------
if(PlugInParamSetTable[TempParamSet] == 0)
    return;

// Skip If Window Is Not Open
//---------------------------
if(hWndPlugIn[TempParamSet] == 0)
    return;

TempPhysicalTrack = FxPtrSawData->FX_Handler_DWord_Param2;

// Verify A Track Match For This Parameter Set
//--------------------------------------------
if(TempPhysicalTrack != PlugInTrack[TempParamSet])
    return;

// Display Automation Changes
//---------------------------
TempCtrlId = FxPtrSawData->FX_Handler_DWord_Param3;
TempDataValue = FxPtrSawData->FX_Handler_Int_Param1;

if(TempCtrlId == AUTO_BYPASS_SW)
    {
    if(TempDataValue)
        DisplayBitmapBtn(PlugInDC[TempParamSet], &RectPlugInBypassDisplayBtn, hPlugInBypassBtnBitmap);
    else
        RefreshBitmap(PlugInDC[TempParamSet], &RectPlugInBypassDisplayBtn, hPlugInMainBitmap);

    return;
    }

if(TempCtrlId == AUTO_VOL_POT)
    {
    DrawVertKnob(PlugInDC[TempParamSet], &RectPlugInVolPot, TempDataValue, 1);
    return;
    }

if(TempCtrlId == AUTO_VALUE_1)
    {
    DisplayPlugInValue(PlugInDC[TempParamSet], AUTO_VALUE_1, TempDataValue, 1);
    return;
    }

if(TempCtrlId == AUTO_VALUE_2)
    {
    DisplayPlugInValue(PlugInDC[TempParamSet], AUTO_VALUE_2, TempDataValue, 1);
    return;
    }

if(TempCtrlId == AUTO_VALUE_3)
    {
    DisplayPlugInValue(PlugInDC[TempParamSet], AUTO_VALUE_3, TempDataValue, 1);
    return;
    }

if(TempCtrlId == AUTO_VALUE_4)
    {
    DisplayPlugInValue(PlugInDC[TempParamSet], AUTO_VALUE_4, TempDataValue, 1);
    return;
    }

if(TempCtrlId == AUTO_VALUE_5)
    {
    DisplayPlugInValue(PlugInDC[TempParamSet], AUTO_VALUE_5, TempDataValue, 1);
    return;
    }

if(TempCtrlId == AUTO_VALUE_6)
    {
    DisplayPlugInValue(PlugInDC[TempParamSet], AUTO_VALUE_6, TempDataValue, 1);
    return;
    }

return;
}

//======================
// Initialize Procedure
//======================

void    __stdcall   InitializeProc(void)

{
WNDCLASS    TempWndClass;
DWORD       TempCount;

// Get Screen Size
//----------------
ScreenMaxH = GetSystemMetrics(SM_CXSCREEN);
ScreenMaxV = GetSystemMetrics(SM_CYSCREEN);

// Register Message Window Class
//------------------------------
ZeroMemory((LPSTR)&TempWndClass, sizeof(WNDCLASS));

TempWndClass.style = CS_BYTEALIGNCLIENT | CS_SAVEBITS;
TempWndClass.lpfnWndProc = (WNDPROC)DLLWndMsgProc;
TempWndClass.cbClsExtra = 0;
TempWndClass.cbWndExtra = 0;
TempWndClass.hInstance = hInstFx;
TempWndClass.hIcon = NULL;
TempWndClass.hCursor = LoadCursor(NULL, IDC_ARROW);
TempWndClass.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
TempWndClass.lpszMenuName = NULL;
TempWndClass.lpszClassName = szMsgClass;

RegisterClass(&TempWndClass);

// Register PlugIn Window Class (Include DWord Of Extra Data)
//-----------------------------------------------------------
ZeroMemory((LPSTR)&TempWndClass, sizeof(WNDCLASS));

TempWndClass.style = CS_BYTEALIGNCLIENT | CS_OWNDC;
TempWndClass.lpfnWndProc = (WNDPROC)DLLWndPlugInProc;
TempWndClass.cbClsExtra = 0;
TempWndClass.cbWndExtra = 4;
TempWndClass.hInstance = hInstFx;
TempWndClass.hIcon = NULL;
TempWndClass.hCursor = LoadCursor(NULL, IDC_ARROW);
TempWndClass.hbrBackground = (HBRUSH)GetStockObject(DKGRAY_BRUSH);
TempWndClass.lpszMenuName = NULL;
TempWndClass.lpszClassName = szPlugInClass;

RegisterClass(&TempWndClass);

// Init Arrays
//------------
for(TempCount = 0; TempCount < MAX_PARAM_SETS; TempCount++)
    {
    hWndPlugIn[TempCount] = 0;
    PlugInDC[TempCount] = 0;
     
    PlugInXStart[TempCount] = 0;
    PlugInYStart[TempCount] = 0;
    PlugInWidth[TempCount] = 0;
    PlugInHeight[TempCount] = 0;
    }

// Load BitMap Images
//-------------------
hPlugInMainBitmap = LoadBitmap(hInstFx, szPlugInMainName);
hPlugInBypassBtnBitmap = LoadBitmap(hInstFx, szPlugInBypassBtnName);
hPlugInMeterBitmap = LoadBitmap(hInstFx, szPlugInMeterName);

// Save Bitmap Header Info
//------------------------
GetObject(hPlugInMainBitmap, sizeof(BITMAP), (LPSTR)&PlugInMainBitmapHdr);
GetObject(hPlugInBypassBtnBitmap, sizeof(BITMAP), (LPSTR)&PlugInBypassBtnBitmapHdr);
GetObject(hPlugInMeterBitmap, sizeof(BITMAP), (LPSTR)&PlugInMeterBitmapHdr);

// Create Drawing Tools
//---------------------
hBlackBrush = CreateSolidBrush(RGB(0, 0, 0));
hGrayBrush = CreateSolidBrush(RGB(192, 192, 192));

hVertKnobPen = CreatePen(PS_SOLID, 1, RGB(255, 246, 137));

hBlackColor = RGB(0, 0, 0);
hTextColor = RGB(255, 246, 137);

// Register Execute Function Message                               
//----------------------------------
SawFxExecuteFunctionMsg = RegisterWindowMessage(szSawFxExecuteFunctionMsg);

// Allocate Work Buffers
//----------------------
PtrBuffWork1 = (LPBYTE)GlobalAlloc(GMEM_FIXED, BuffWorkSize);

if(PtrBuffWork1 == NULL)
    MemoryAllocFailureFlag = 1;

// Get Current Module Path
//------------------------
IQSGetModuleDirectory(szOrgPath);

// Load Special Cursors
//---------------------
hPlusMinusCursor = LoadCursor(hInstFx, szPlugInPlusMinusCursorName);

// Init Threads
//-------------
InitThreads();

return;
}

//====================
// Shutdown Procedure
//====================

void    __stdcall   ShutdownProc(void)

{
DWORD   TempCount;

// Close All Open Windows
//-----------------------
for(TempCount = 0; TempCount < MAX_PARAM_SETS; TempCount++)
    {
    if(hWndPlugIn[TempCount])
        DestroyPlugInWindow(TempCount);
    }

// ShutDown Threads
//-----------------
ShutDownThreads();

// Delete Bitmaps
//---------------
DeleteObject(hPlugInMainBitmap);
DeleteObject(hPlugInBypassBtnBitmap);
DeleteObject(hPlugInMeterBitmap);

// Free Buffers
//-------------
if(PtrBuffWork1)
    GlobalFree(PtrBuffWork1);

// Delete Drawing Tools
//---------------------
DeleteObject(hBlackBrush);
DeleteObject(hGrayBrush);

DeleteObject(hVertKnobPen);

// Destroy Special Cursors
//------------------------
DestroyCursor(hPlusMinusCursor);

return;
}

//==============
// Init Threads
//==============

void    __stdcall   InitThreads(void)

{
// Init Background Thread
//-----------------------
hFxBackThread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)FxBackServiceCallback ,NULL, CREATE_SUSPENDED, (LPDWORD)&FxBackThreadID);
SetThreadPriority(hFxBackThread, THREAD_PRIORITY_NORMAL);
return;
}

//==================
// ShutDown Threads
//==================

void    __stdcall   ShutDownThreads(void)

{
ForceThreadExit(hFxBackThread, FxBackThreadID, &FxBackThreadExitFlag);
return;
}

//===================
// Force Thread Exit
//===================

void    __stdcall   ForceThreadExit(HANDLE TempThreadHandle, DWORD TempThreadID, PDWORD TempExitFlagPtr)

{
DWORD   TempExitCode;

if(TempThreadHandle == 0)
    return;

// Set The Exit Flag And Resume The Thread
//----------------------------------------
*TempExitFlagPtr = 1;
ResumeThread(TempThreadHandle);

TempExitCode = STILL_ACTIVE;

// Wait Till Thread Exits
//-----------------------
while(TempExitCode == STILL_ACTIVE)
    {
    GetExitCodeThread(TempThreadHandle, &TempExitCode);
    Sleep(1);
    }

// Close The Thread Handle
//------------------------
CloseHandle(TempThreadHandle);
return;
}

//====================================
// Background Service Thread Callback
//====================================

DWORD   WINAPI      FxBackServiceCallback(LPVOID lParam)

{
// Loop Until Suspended Or Forced To Exit
//---------------------------------------
while(TRUE)
    {
    // Force Thread Exit
    //------------------
    if(FxBackThreadExitFlag)
        break;

    // Process Thread Actions
    //-----------------------
    PlusMinusCheck();

    if(FxBackThreadSuspendFlag)
        {
        FxBackThreadSuspendFlag = 0;
        SuspendThread(hFxBackThread);
        }
    else
        Sleep(FxBackThreadSleepTime);
    }

return(0);
}

//==========================
// IQS Get Module Directory
//==========================

void    __stdcall   IQSGetModuleDirectory(LPSTR TempPath)

{
// Get Full Name And Path Of Module
//---------------------------------
GetModuleFileName(hInstFx, TempPath, MAX_PATH);

// Extract The Path
//-----------------
ExtractPath(TempPath);
return;
}

//================================
// Extract Drive & Path From Name
//================================

void    __stdcall   ExtractPath(LPSTR PtrPathBuffer)

{
int     TempLen;
int     TempCount;

// Get String Length
//------------------
TempLen = lstrlen(PtrPathBuffer);

// Force To C: If String Length Is Zero
//-------------------------------------
if(TempLen == 0)
    {
    *PtrPathBuffer = 'C';
    *(PtrPathBuffer + 1) = ':';
    TempLen = 2;
    goto AddBackSlash;
    }
    
// Step Through String Backwards to Find The Last Backslash
//---------------------------------------------------------
for(TempCount = TempLen - 1; TempCount > 0; TempCount = TempCount - 1)
    {
    if(*(PtrPathBuffer + TempCount) == 92)
        {
        *(PtrPathBuffer + TempCount + 1) = 0;
        TempLen = TempCount + 1;
        break;
        }
    }

// Add Final Backslash If Needed
//------------------------------
AddBackSlash:
PtrPathBuffer = PtrPathBuffer + (TempLen - 1);
if(*PtrPathBuffer != 92)
    {
    PtrPathBuffer = PtrPathBuffer + 1;
    *PtrPathBuffer = 92;
    PtrPathBuffer = PtrPathBuffer + 1;
    *PtrPathBuffer = 0;
    }

return;
}

//===========================
// IQS Get Current Directory
//===========================

void    __stdcall   IQSGetCurDirectory(LPSTR TempPath)

{
// Get Current Directory
//----------------------
GetCurrentDirectory(MAX_PATH, TempPath);

// Add Final Backslash If Needed
//------------------------------
AddBackSlashToPath(TempPath);
return;
}

//=======================
// Add BackSlash To Path
//=======================

void    __stdcall   AddBackSlashToPath(LPSTR PtrPathBuffer)

{
int     TempLen;

TempLen = lstrlen(PtrPathBuffer);

if(TempLen == 0)
    TempLen = 1;

// Add Final Backslash If Needed
//------------------------------
PtrPathBuffer = PtrPathBuffer + (TempLen - 1);
if(*PtrPathBuffer != 92)
    {
    PtrPathBuffer = PtrPathBuffer + 1;
    *PtrPathBuffer = 92;
    PtrPathBuffer = PtrPathBuffer + 1;
    *PtrPathBuffer = 0;
    }

return;
}

//=============================
// Create About Message Window
//=============================

void    __stdcall   CreateAboutWindow(void)

{
MSG     Msg;
int     TempWidth;
int     TempHeight;
HDC     hDC;
RECT    TempRect;

TempWidth = 300;
TempHeight = 170;

GetWindowRect(hWndPlugIn[FxActiveParamSet], &TempRect);

hWndAbout = CreateWindow(
                szMsgClass,
                "About",
                WS_POPUP        |
                WS_CAPTION      |
                WS_BORDER       |
                WS_SYSMENU      |
                WS_VISIBLE      |
                WS_CLIPCHILDREN |
                WS_CLIPSIBLINGS,

                TempRect.left,
                TempRect.top,
                TempWidth,
                TempHeight,

                hWndPlugIn[FxActiveParamSet],
                NULL,
                hInstFx,
                NULL);

if(hWndAbout == NULL)
    return;

hDC = GetDC(hWndAbout);
UpdateWindow(hWndAbout);

// Modal Processing Loop
//----------------------
AboutModalFlag = 1;

// Loop Here Until A Key Or Mouse Button Is Pressed
//-------------------------------------------------
while(AboutModalFlag)
    {
    if(PeekMessage(&Msg, NULL, 0, 0, PM_REMOVE))
        {
        if(Msg.message == WM_PAINT)
            goto Dispatch;

        if(Msg.message == WM_KEYDOWN)
            break;

        if(Msg.message == WM_LBUTTONDOWN)
            break;

        if(Msg.message == WM_RBUTTONDOWN)
            break;

        if(Msg.message == WM_NCLBUTTONDOWN)
            break;

        continue;

        Dispatch:
        TranslateMessage(&Msg);
        DispatchMessage(&Msg);
        }
    }

AboutModalFlag = 0;
DestroyWindow(hWndAbout);
hWndAbout = 0;
return;
}

//====================
// Paint About Window
//====================

void    __stdcall   PaintAboutWindow(HDC hDC)

{
RECT    rRect;

// Fill Background With Gradient
//------------------------------
GetClientRect(hWndAbout, &rRect);
GradientFillRect(hDC, &rRect, rRect.bottom, 240, 240, 240);

// Display Text Info
//------------------
SelectObject(hDC, hIQSSysFont);
SetBkMode(hDC, TRANSPARENT);

rRect.top = rRect.top + 10;
DrawText(hDC, AboutMsg, -1, &rRect, DT_CENTER | DT_WORDBREAK);
return;
}

//====================================================
// Msg Class Window Procedure For Processing Messages
//====================================================

LRESULT CALLBACK    DLLWndMsgProc(HWND hWnd, UINT wMessage, WPARAM wParam, LPARAM lParam)

{
HDC         hDC;
PAINTSTRUCT psPaint;

switch(wMessage)
    {
    case WM_PAINT:
        hDC = BeginPaint(hWnd, &psPaint);

        if(hWnd == hWndAbout)
            PaintAboutWindow(hDC);
        
        EndPaint(hWnd, &psPaint);
        break;

    default:
        return DefWindowProc(hWnd, wMessage, wParam, lParam);
    }

return(0);
}

//==================
// Check Boundaries
//==================

int __stdcall   CheckBoundaries(LPRECT RectAddr)

{
RECT    TempRect;

// Check If Mouse Is Inside Rectangle Boundaries
//----------------------------------------------
CopyRect(&TempRect, RectAddr);

if(CurMouseX < TempRect.left)
    goto OutOfBounds;

if(CurMouseX > TempRect.right)
    goto OutOfBounds;

if(CurMouseY < TempRect.top)
    goto OutOfBounds;

if(CurMouseY > TempRect.bottom)
    goto OutOfBounds;

// Yes It Is
//----------
return(1);

// No It Is Not
//-------------
OutOfBounds:
return(0);
}

//==========================================
// Pick Open FileName
//
// Input:   szStringBuff = Display Text
//
// Output:  szOpenName = Selected Path/Name
//          ofFileName = Return Info
//==========================================

BOOL    __stdcall   PickOpenFileName(LPSTR szTempStartName, LPSTR szTempStartPath, LPSTR szTempFilterList, int TempFilterIndex, LPSTR szTempDefExt)

{
BOOL    TempResult;

if(szTempStartName == NULL)
    szOpenName[0] = 0;
else
    lstrcpy(szOpenName, szTempStartName);

lstrcpy(szAppendExt, szTempDefExt);

if(szAppendExt[1] == '*')
    szAppendExt[0] = 0;

ZeroMemory((LPSTR)&ofFileName, sizeof(OPENFILENAME));

ofFileName.lStructSize = sizeof(OPENFILENAME);
ofFileName.hwndOwner = hWndSawMain;
ofFileName.lpstrFilter = szTempFilterList;
ofFileName.lpstrCustomFilter = NULL;
ofFileName.nFilterIndex = TempFilterIndex;
ofFileName.lpstrFile = (LPSTR)szOpenName;
ofFileName.nMaxFile = MAX_PATH;
ofFileName.lpstrFileTitle = NULL;
ofFileName.nMaxFileTitle = 0;
ofFileName.lpstrInitialDir = (LPSTR)szTempStartPath;
ofFileName.lpstrTitle = (LPSTR)szStringBuff;
ofFileName.Flags = OFN_HIDEREADONLY | OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST;
ofFileName.lpstrDefExt = szAppendExt;

// Display File Dialog Box
//------------------------
TempResult = GetOpenFileName(&ofFileName);

return(TempResult);
}

//==========================================
// Pick Save FileName
//
// Input:   szStringBuff = Display Text
//
// Output:  szSaveName = Selected Path/Name
//          ofFileName = Return Info
//==========================================

BOOL    __stdcall   PickSaveFileName(LPSTR szTempStartName, LPSTR szTempStartPath, LPSTR szTempFilterList, int TempFilterIndex, LPSTR szTempDefExt, DWORD TempOverWriteFlag)

{
BOOL    TempResult;

if(szTempStartName == NULL)
    szSaveName[0] = 0;
else
    lstrcpy(szSaveName, szTempStartName);

lstrcpy(szAppendExt, szTempDefExt);

if(szAppendExt[1] == '*')
    szAppendExt[0] = 0;

ZeroMemory((LPSTR)&ofFileName, sizeof(OPENFILENAME));

ofFileName.lStructSize = sizeof(OPENFILENAME);
ofFileName.hwndOwner = hWndSawMain;
ofFileName.lpstrFilter = szTempFilterList;
ofFileName.lpstrCustomFilter = NULL;
ofFileName.nFilterIndex = TempFilterIndex;
ofFileName.lpstrFile = (LPSTR)szSaveName;
ofFileName.nMaxFile = MAX_PATH;
ofFileName.lpstrFileTitle = NULL;
ofFileName.nMaxFileTitle = 0;
ofFileName.lpstrInitialDir = (LPSTR)szTempStartPath;
ofFileName.lpstrTitle = (LPSTR)szStringBuff;

if(TempOverWriteFlag)
    ofFileName.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT;
else
    ofFileName.Flags = OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;

ofFileName.lpstrDefExt = szAppendExt;

// Display File Dialog Box
//------------------------
TempResult = GetSaveFileName(&ofFileName);

return(TempResult);
}

//================
// Refresh Bitmap
//================

void    __stdcall   RefreshBitmap(HDC hDC, LPRECT RectAddr, HBITMAP hBitmap)

{
RECT    TempRect;
HDC     TempMemDC;
int     TempWidth;
int     TempHeight;

// Refresh Current Bitmap Within Rectangle Boundaries
//---------------------------------------------------
CopyRect(&TempRect, RectAddr);
TempWidth = TempRect.right - TempRect.left + 1;
TempHeight = TempRect.bottom - TempRect.top + 1;

TempMemDC = CreateCompatibleDC(hDC);
SelectObject(TempMemDC, hBitmap);

BitBlt(hDC, TempRect.left, TempRect.top, TempWidth, TempHeight, TempMemDC, TempRect.left, TempRect.top, SRCCOPY);

DeleteDC(TempMemDC);
return;
}

//=======================
// Display Bitmap Button
//=======================

void    __stdcall   DisplayBitmapBtn(HDC hDC, LPRECT RectAddr, HBITMAP hBitmapBtn)

{
RECT    TempRect;
HDC     TempMemDC;
int     TempWidth;
int     TempHeight;

// Display Button Bitmap Within Rectangle Boundaries
//--------------------------------------------------
CopyRect(&TempRect, RectAddr);
TempWidth = TempRect.right - TempRect.left + 1;
TempHeight = TempRect.bottom - TempRect.top + 1;

TempMemDC = CreateCompatibleDC(hDC);
SelectObject(TempMemDC, hBitmapBtn);

BitBlt(hDC, TempRect.left, TempRect.top, TempWidth, TempHeight, TempMemDC, 0, 0, SRCCOPY);

DeleteDC(TempMemDC);
return;
}

//====================
// Check Virtual Keys
//====================

DWORD   __stdcall   ChkVKey(DWORD VKeyType)

{
int     nKeyState;

// Check If Virtual Key Is Pressed
//--------------------------------
nKeyState = GetAsyncKeyState(VKeyType);

if(HIWORD(nKeyState))
    return(1);

return(0);    
}

//=================
// IQS Move Window
//=================

void    __stdcall   IQSMoveWindow(HWND hWnd)

{
POINT   TempPt;
LPARAM  lParam;

// Fake A Mouse Press To The TitleBar Area
//----------------------------------------
TempPt.x = CurMouseX;
TempPt.y = CurMouseY;
ClientToScreen(hWnd, &TempPt);

lParam = MAKELPARAM(TempPt.x, TempPt.y);

SendMessage (hWnd, WM_NCLBUTTONDOWN, HTCAPTION, lParam);
return;
}

//=========================
// Gradient Fill Rectangle
//=========================

void    __stdcall   GradientFillRect(HDC hDC, LPRECT RectAddr, int RectBottom, int ColorR, int ColorG, int ColorB)

{
RECT    rRect;
int     TempValue;
int     TempValue2;
int     ColorCount = 64;
int     ColorDec;
HBRUSH  hBrush;
int     iCount;

// Generate A Gradient Of The Specified Color To Fill The Rectangle Boundaries
//----------------------------------------------------------------------------
CopyRect(&rRect, RectAddr);

TempValue = rRect.bottom / ColorCount;
TempValue = TempValue + 1;
rRect.bottom = rRect.top + TempValue;

TempValue2 = 0;

if(ColorR > TempValue2)
    TempValue2 = ColorR;

if(ColorG > TempValue2)
    TempValue2 = ColorG;

if(ColorB > TempValue2)
    TempValue2 = ColorB;

ColorDec = TempValue2 / ColorCount;

for(iCount = 0; iCount < ColorCount; iCount++)
    {
    hBrush = CreateSolidBrush(RGB(ColorR, ColorG, ColorB));
    FillRect(hDC, &rRect, hBrush);
    DeleteObject(hBrush);

    rRect.top = rRect.bottom;
    rRect.bottom = rRect.top + TempValue;

    if(ColorR > ColorDec)
        ColorR = ColorR - ColorDec;

    if(ColorG > ColorDec)
        ColorG = ColorG - ColorDec;

    if(ColorB > ColorDec)
        ColorB = ColorB - ColorDec;

    if(rRect.bottom > RectBottom)
        {
        rRect.bottom = RectBottom;
        hBrush = CreateSolidBrush(RGB(ColorR, ColorG, ColorB));
        FillRect(hDC, &rRect, hBrush);
        DeleteObject(hBrush);
        break;
        }
    }

return;
}

//======================
// Clear All Mouse Msgs
//======================

void    __stdcall   ClearAllMouseMsgs(void)

{
MSG     msg;

// Clear Any Mouse Msg
//--------------------
ClearKeys:
if(PeekMessage(&msg, NULL, WM_MOUSEFIRST, WM_MOUSELAST, PM_NOYIELD | PM_REMOVE))
    goto ClearKeys;
    
return;
}

//=========================
// Wait For Event For mSec
//=========================

int     __stdcall   WaitMsecForEvent(int DelayTime, UINT Event)

{
DWORD   TempStartMs;
DWORD   TempEndMs;
DWORD   TempCurMs;
MSG     msg;

// Get Start Ms
//-------------
TempStartMs = GetTickCount();
TempEndMs = TempStartMs + DelayTime;

// Check For WrapAround Condition
//-------------------------------
if(TempEndMs < TempStartMs)
    {
    WaitWrapAround:
    TempCurMs = GetTickCount();
    
    if(TempCurMs > TempStartMs)
        goto WaitWrapAround;
    }

// Wait Specified Amount
//----------------------
WaitLoop:
TempCurMs = GetTickCount();

// Look For Specified Event
//-------------------------
if(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
    {
    // Match
    //------
    if(msg.message == Event)
        return(1);
    }

if(TempCurMs < TempEndMs)
    goto WaitLoop;

// No Match
//---------
return(0);
}

//======================
// Create PlugIn Window
//======================

void    __stdcall   CreatePlugInWindow(DWORD TempParamSet, HWND hWndRefWindow)

{
RECT    TempRect;

// If Window Is Already Opened Then Just Refresh It
//-------------------------------------------------
if(hWndPlugIn[TempParamSet])
    {
    InvalidateRect(hWndPlugIn[TempParamSet], NULL, FALSE);
    UpdateWindow(hWndPlugIn[TempParamSet]);
    return;
    }

// If This Is The First Time Then Use The Reference Window For Positioning
// Or Center It If There Is No Reference Window Handle
//------------------------------------------------------------------------
if(PlugInWidth[TempParamSet] == 0)
    {
    PlugInWidth[TempParamSet] = PlugInMainBitmapHdr.bmWidth;
    PlugInHeight[TempParamSet] = PlugInMainBitmapHdr.bmHeight;

    if(hWndRefWindow)
        {
        GetWindowRect(hWndRefWindow, &TempRect);
        PlugInXStart[TempParamSet] = TempRect.left + 40;
        PlugInYStart[TempParamSet] = TempRect.top + 40;
        }
    else
        {
        PlugInXStart[TempParamSet] = (ScreenMaxH - PlugInWidth[TempParamSet]) / 2;
        PlugInYStart[TempParamSet] = 80;
        }

    if(PlugInXStart[TempParamSet] + PlugInWidth[TempParamSet] > ScreenMaxH)
        PlugInXStart[TempParamSet] = ScreenMaxH - PlugInWidth[TempParamSet] - 4;
         
    if(PlugInYStart[TempParamSet] + PlugInHeight[TempParamSet] > ScreenMaxV)
        PlugInYStart[TempParamSet] = ScreenMaxV - PlugInHeight[TempParamSet] - 4;
    }

// Create Window
//--------------
hWndPlugIn[TempParamSet] = CreateWindowEx(
                WS_EX_TOPMOST,
                szPlugInClass,
                "",

                WS_POPUP        |
                WS_CLIPCHILDREN,

                PlugInXStart[TempParamSet],
                PlugInYStart[TempParamSet],
                PlugInWidth[TempParamSet],
                PlugInHeight[TempParamSet],
                
                hWndSawMain,
                NULL,
                hInstFx,
                NULL);

if(hWndPlugIn[TempParamSet] == NULL)
    return;
                   
// Set ParamSet Value As Extra Data
//---------------------------------
SetWindowLong(hWndPlugIn[TempParamSet], 0, TempParamSet);

// Set DC Info
//------------
PlugInDC[TempParamSet] = GetDC(hWndPlugIn[TempParamSet]);
SetBkMode(PlugInDC[TempParamSet], TRANSPARENT);
SelectObject(PlugInDC[TempParamSet], hIQSSysFont);

// Display The Window And Give It The Focus
//-----------------------------------------
ShowWindow(hWndPlugIn[TempParamSet], SW_SHOW);
UpdateWindow(hWndPlugIn[TempParamSet]);
SetFocus(hWndPlugIn[TempParamSet]);
return;
}

//=======================
// Destroy PlugIn Window
//=======================

void    __stdcall   DestroyPlugInWindow(DWORD TempParamSet)

{
RECT    TempRect;

// Skip If Window Is Already Destroyed
//------------------------------------
if(hWndPlugIn[TempParamSet] == 0)
    return;
    
// Save The Last Position Info
//----------------------------
GetWindowRect(hWndPlugIn[TempParamSet], &TempRect);
PlugInXStart[TempParamSet] = TempRect.left;
PlugInYStart[TempParamSet] = TempRect.top;
PlugInWidth[TempParamSet] = TempRect.right - TempRect.left;
PlugInHeight[TempParamSet] = TempRect.bottom - TempRect.top;

// Destroy Window
//---------------
ReleaseDC(hWndPlugIn[TempParamSet], PlugInDC[TempParamSet]);
DestroyWindow(hWndPlugIn[TempParamSet]);

hWndPlugIn[TempParamSet] = 0;
PlugInDC[TempParamSet] = 0;
return;
}

//=================================================
// PlugIn Window Procedure For Processing Messages
//=================================================

LRESULT CALLBACK    DLLWndPlugInProc(HWND hWnd, UINT wMessage, WPARAM wParam, LPARAM lParam)

{
HDC         hDC;
PAINTSTRUCT psPaint;
HDC         TempMemoryDC;

// Query Window For Parameter Set Value
//-------------------------------------
FxActiveParamSet = GetWindowLong(hWnd, 0);

switch(wMessage)
    {
    case WM_PAINT:
        hDC = BeginPaint(hWnd, &psPaint);

        // Put Bitmap Onscreen
        //--------------------
        TempMemoryDC = CreateCompatibleDC(hDC);
        SelectObject(TempMemoryDC, hPlugInMainBitmap);
        BitBlt(hDC, 0, 0, PlugInMainBitmapHdr.bmWidth, PlugInMainBitmapHdr.bmHeight, TempMemoryDC, 0, 0, SRCCOPY);
        DeleteDC(TempMemoryDC);

        // Display Track Number
        //---------------------
        SetTextColor(hDC, hTextColor);

        if(PlugInTrack[FxActiveParamSet] == NO_ASSIGNMENT)
            lstrcpy(szWorkBuff, "- - -");
        else
            {        
            if(PlugInTrack[FxActiveParamSet] < FxReturnTrkOffset)
                {
                wsprintf(szWorkBuff, "In %02u", PlugInTrack[FxActiveParamSet] + 1);
                goto DisplayTrack;
                }

            if(PlugInTrack[FxActiveParamSet] < FxOutputTrkOffset)
                {
                wsprintf(szWorkBuff, "Rtn %02u", PlugInTrack[FxActiveParamSet] - FxReturnTrkOffset + 1);
                goto DisplayTrack;
                }

            wsprintf(szWorkBuff, "Out %02u", PlugInTrack[FxActiveParamSet] - FxOutputTrkOffset + 1);
            }
        
        DisplayTrack:
        DrawText(hDC, szWorkBuff, -1, &RectPlugInTrack, DT_SINGLELINE | DT_CENTER | DT_VCENTER | DT_WORDBREAK);

        // Preset Buttons
        //---------------
        PresetPlugInBtns(0);

        // Draw Knobs
        //-----------
        DrawVertKnob(hDC, &RectPlugInVolPot, PlugInDataValue[FxActiveParamSet][AUTO_VOL_POT], 0);

        // Display Values
        //---------------
        DisplayPlugInValue(hDC, AUTO_VALUE_1, PlugInDataValue[FxActiveParamSet][AUTO_VALUE_1], 0);
        DisplayPlugInValue(hDC, AUTO_VALUE_2, PlugInDataValue[FxActiveParamSet][AUTO_VALUE_2], 0);
        DisplayPlugInValue(hDC, AUTO_VALUE_3, PlugInDataValue[FxActiveParamSet][AUTO_VALUE_3], 0);
        DisplayPlugInValue(hDC, AUTO_VALUE_4, PlugInDataValue[FxActiveParamSet][AUTO_VALUE_4], 0);
        DisplayPlugInValue(hDC, AUTO_VALUE_5, PlugInDataValue[FxActiveParamSet][AUTO_VALUE_5], 0);
        DisplayPlugInValue(hDC, AUTO_VALUE_6, PlugInDataValue[FxActiveParamSet][AUTO_VALUE_6], 0);

        // Display Labels
        //---------------
        SetTextColor(hDC, hBlackColor);
        DrawText(hDC, PlugInParamLabel[0], -1, &RectPlugInLabel1, DT_SINGLELINE | DT_CENTER | DT_VCENTER);
        DrawText(hDC, PlugInParamLabel[1], -1, &RectPlugInLabel2, DT_SINGLELINE | DT_CENTER | DT_VCENTER);
        DrawText(hDC, PlugInParamLabel[2], -1, &RectPlugInLabel3, DT_SINGLELINE | DT_CENTER | DT_VCENTER);
        DrawText(hDC, PlugInParamLabel[3], -1, &RectPlugInLabel4, DT_SINGLELINE | DT_CENTER | DT_VCENTER);
        DrawText(hDC, PlugInParamLabel[4], -1, &RectPlugInLabel5, DT_SINGLELINE | DT_CENTER | DT_VCENTER);
        DrawText(hDC, PlugInParamLabel[5], -1, &RectPlugInLabel6, DT_SINGLELINE | DT_CENTER | DT_VCENTER);

        EndPaint(hWnd, &psPaint);
        break;

    case WM_SETCURSOR:
        // Maintain Plus-Minus Cursor If Needed
        //-------------------------------------
        if(PlusMinusActive)
            break;

        goto DefProc;
        break;

    case WM_CLOSE:
        DestroyPlugInWindow(FxActiveParamSet);
        break;

    case WM_LBUTTONDOWN:
        CurMouseX = LOWORD(lParam);
        CurMouseY = HIWORD(lParam);

        // Check If Mouse Click Is In Any Active Zones
        //--------------------------------------------
        CheckPlugInZones();
        break;
    
    case WM_LBUTTONUP:
        ClipCursor(NULL);

        // End Plus-Minus If Needed
        //-------------------------
        if(PlusMinusActive)
            PlusMinusEnd();

        // End Vertical Knob Motion If Needed
        //-----------------------------------
        if(VertKnobActive)    
            {
            // Set New Value
            //--------------
            SetVertKnobValue();

            VertKnobActive = 0;
            VertKnobCtrlId = NO_VALUE;
            SendMessage(hWndSawMain, SawFxExecuteFunctionMsg, SET_EDL_MODIFY_FLAG, 0);
            }

        break;
        
    case WM_RBUTTONDOWN:
        CurMouseX = LOWORD(lParam);
        CurMouseY = HIWORD(lParam);

        // Set To Default Value If Needed
        //-------------------------------
        if(PlusMinusActive)
            {
            PlusMinusActive = 0;
            FxBackThreadSuspendFlag = 1;
            ActionSetToDefaultValue();
            break;
            }

        if(VertKnobActive)    
            {
            SetVertKnobValueDef();
            break;
            }

        // Check For Right-Click Active Zones
        //-----------------------------------
        if(CheckPlugInZonesRight())
            break;

        // Start Or Stop Playback As Needed
        //---------------------------------
        if(FxProcessFlag == REALTIME_PLAYBACK)
            SendMessage(hWndSawMain, SawFxExecuteFunctionMsg, STOP_MT_AUDIO, 0);
        else
            SendMessage(hWndSawMain, SawFxExecuteFunctionMsg, START_MT_AUDIO_PLAY, 0);

        break;

    case WM_MOUSEMOVE:
        CurMouseX = LOWORD(lParam);
        CurMouseY = HIWORD(lParam);

        // Adjust Knob Value
        //------------------
        if(VertKnobActive)
            {
            MoveVertKnob();

            if(FxProcessFlag == REALTIME_PLAYBACK)
                SetVertKnobValue();
            }

        break;

    DefProc:
    default:
        return DefWindowProc(hWnd, wMessage, wParam, lParam);
    }

return (0L);
}

//======================
// Display PlugIn Value
//======================

void    __stdcall   DisplayPlugInValue(HDC hDC, DWORD TempCtrlId, int TempDataValue, DWORD RefreshFlag)

{
RECT    TempRect;

// Skip If Window Is Not Open
//---------------------------
if(hWndPlugIn[FxActiveParamSet] == 0)
    return;

// Display Proper Control Value
//-----------------------------
if(TempCtrlId == AUTO_VALUE_1)
    {
    CopyRect(&TempRect, &RectPlugInValue1);

    if(RefreshFlag)
        RefreshBitmap(hDC, &TempRect, hPlugInMainBitmap);

    lstrcpy(szWorkBuff, OutputFormatTable[TempDataValue]);

    SetTextColor(hDC, hTextColor);
    DrawText(hDC, szWorkBuff, -1, &TempRect, DT_SINGLELINE | DT_CENTER | DT_VCENTER | DT_WORDBREAK);
    return;
    }

if(TempCtrlId == AUTO_VALUE_2)
    {
    CopyRect(&TempRect, &RectPlugInValue2);

    if(RefreshFlag)
        RefreshBitmap(hDC, &TempRect, hPlugInMainBitmap);

    lstrcpy(szWorkBuff, MeterResponseTable[TempDataValue]);

    SetTextColor(hDC, hTextColor);
    DrawText(hDC, szWorkBuff, -1, &TempRect, DT_SINGLELINE | DT_CENTER | DT_VCENTER | DT_WORDBREAK);
    return;
    }

if(TempCtrlId == AUTO_VALUE_3)
    {
    CopyRect(&TempRect, &RectPlugInValue3);

    if(RefreshFlag)
        RefreshBitmap(hDC, &TempRect, hPlugInMainBitmap);

    wsprintf(szWorkBuff, "%u %%", TempDataValue);

    SetTextColor(hDC, hTextColor);
    DrawText(hDC, szWorkBuff, -1, &TempRect, DT_SINGLELINE | DT_CENTER | DT_VCENTER | DT_WORDBREAK);
    return;
    }

if(TempCtrlId == AUTO_VALUE_4)
    {
    CopyRect(&TempRect, &RectPlugInValue4);

    if(RefreshFlag)
        RefreshBitmap(hDC, &TempRect, hPlugInMainBitmap);

    wsprintf(szWorkBuff, "%u %%", TempDataValue);

    SetTextColor(hDC, hTextColor);
    DrawText(hDC, szWorkBuff, -1, &TempRect, DT_SINGLELINE | DT_CENTER | DT_VCENTER | DT_WORDBREAK);
    return;
    }

if(TempCtrlId == AUTO_VALUE_5)
    {
    CopyRect(&TempRect, &RectPlugInValue5);

    if(RefreshFlag)
        RefreshBitmap(hDC, &TempRect, hPlugInMainBitmap);

    wsprintf(szWorkBuff, "%u %%", TempDataValue);

    SetTextColor(hDC, hTextColor);
    DrawText(hDC, szWorkBuff, -1, &TempRect, DT_SINGLELINE | DT_CENTER | DT_VCENTER | DT_WORDBREAK);
    return;
    }

if(TempCtrlId == AUTO_VALUE_6)
    {
    CopyRect(&TempRect, &RectPlugInValue6);

    if(RefreshFlag)
        RefreshBitmap(hDC, &TempRect, hPlugInMainBitmap);

    wsprintf(szWorkBuff, "%u %%", TempDataValue);

    SetTextColor(hDC, hTextColor);
    DrawText(hDC, szWorkBuff, -1, &TempRect, DT_SINGLELINE | DT_CENTER | DT_VCENTER | DT_WORDBREAK);
    return;
    }

return;
}

//=======================
// Preset PlugIn Buttons
//=======================

void    __stdcall   PresetPlugInBtns(DWORD RefreshFlag)

{
// Preset Any Buttons That Are Pushed In
//--------------------------------------
if(RefreshFlag)
    RefreshBitmap(PlugInDC[FxActiveParamSet], &RectPlugInBypassBtn, hPlugInMainBitmap);
    
if(PlugInDataValue[FxActiveParamSet][AUTO_BYPASS_SW])
    DisplayBitmapBtn(PlugInDC[FxActiveParamSet], &RectPlugInBypassDisplayBtn, hPlugInBypassBtnBitmap);

return;
}

//====================
// Check PlugIn Zones
//====================

void    __stdcall   CheckPlugInZones(void)

{
int     TempLRCenter;

// Pop Up Options Menu
//--------------------
if(CheckBoundaries(&RectPlugInOptions))
    {
    if(FxProcessFlag)
        return;

    CreateOptionsSelectionMenu();
    return;
    }

// Close Window
//-------------
if(CheckBoundaries(&RectPlugInCloseBtn))
    {
    DestroyPlugInWindow(FxActiveParamSet);
    return;
    }

// Toggle Bypass Button
//---------------------
if(CheckBoundaries(&RectPlugInBypassBtn))
    {
    if(PlugInDataValue[FxActiveParamSet][AUTO_BYPASS_SW])
        {
        RefreshBitmap(PlugInDC[FxActiveParamSet], &RectPlugInBypassDisplayBtn, hPlugInMainBitmap);
        PlugInDataValue[FxActiveParamSet][AUTO_BYPASS_SW] = 0;
        }
    else
        {
        DisplayBitmapBtn(PlugInDC[FxActiveParamSet], &RectPlugInBypassDisplayBtn, hPlugInBypassBtnBitmap);
        PlugInDataValue[FxActiveParamSet][AUTO_BYPASS_SW] = 1;

        if(FxProcessFlag == REALTIME_PLAYBACK)
            ResetMeterDisplay(FxActiveParamSet);
        }

    SendAutomation(AUTO_BYPASS_SW, PlugInDataValue[FxActiveParamSet][AUTO_BYPASS_SW], 0, 1);

    SendMessage(hWndSawMain, SawFxExecuteFunctionMsg, SET_EDL_MODIFY_FLAG, 0);
    return;
    }

// Ignore Track Zone
//------------------
if(CheckBoundaries(&RectPlugInTrack))
    {
    return;
    }

// Setup To Move Vertical Knob
//----------------------------
if(CheckBoundaries(&RectPlugInVolPot))
    {
    VertKnobActive = 1;
    VertKnobCtrlId = AUTO_VOL_POT;
    VertKnobPos = PlugInDataValue[FxActiveParamSet][AUTO_VOL_POT];
    VertKnobPosDef = 5;
    VertKnobWindow = hWndPlugIn[FxActiveParamSet];
    VertKnobPen = hVertKnobPen;
    VertKnobBackBitmap = hPlugInMainBitmap;
    CopyRect(&VertKnobRect, &RectPlugInVolPot);

    LockToVertKnob();     
    return;
    }

// Latch To Control 1
//-------------------
if(CheckBoundaries(&RectPlugInValue1))
    {
    ActiveControlType = AUTO_VALUE_1;
    ActiveControlValue = PlugInDataValue[FxActiveParamSet][AUTO_VALUE_1];
    ActiveControlValueMin = 0;
    ActiveControlValueMax = 5;
    ActiveControlValueDef = 0;
    ActiveControlMultiplier = 1;
    CopyRect(&ActiveControlRect, &RectPlugInValue1);
    
    ActiveControlListPtr = (LPSTR)OutputFormatTable[0];

    TempLRCenter = ((ActiveControlRect.right - ActiveControlRect.left) / 2) + ActiveControlRect.left;

    if(CurMouseX > TempLRCenter)
        ActionControlScroll();
    else
        ActionControlMenuList();

    return;
    }

// Latch To Control 2
//-------------------
if(CheckBoundaries(&RectPlugInValue2))
    {
    ActiveControlType = AUTO_VALUE_2;
    ActiveControlValue = PlugInDataValue[FxActiveParamSet][AUTO_VALUE_2];
    ActiveControlValueMin = 0;
    ActiveControlValueMax = 2;
    ActiveControlValueDef = 0;
    ActiveControlMultiplier = 1;
    CopyRect(&ActiveControlRect, &RectPlugInValue2);
    
    ActiveControlListPtr = (LPSTR)MeterResponseTable[0];

    TempLRCenter = ((ActiveControlRect.right - ActiveControlRect.left) / 2) + ActiveControlRect.left;

    if(CurMouseX > TempLRCenter)
        ActionControlScroll();
    else
        ActionControlMenuList();

    return;
    }

// Latch To Control 3
//-------------------
if(CheckBoundaries(&RectPlugInValue3))
    {
    ActiveControlType = AUTO_VALUE_3;
    ActiveControlValue = PlugInDataValue[FxActiveParamSet][AUTO_VALUE_3];
    ActiveControlValueMin = 0;
    ActiveControlValueMax = 100;
    ActiveControlValueDef = 0;
    ActiveControlMultiplier = 1;
    CopyRect(&ActiveControlRect, &RectPlugInValue3);
    
    ActiveControlListPtr = (LPSTR)NULL;

    TempLRCenter = ((ActiveControlRect.right - ActiveControlRect.left) / 2) + ActiveControlRect.left;

    if(CurMouseX > TempLRCenter)
        ActionControlScroll();
    else
        ActionControlMenuList();

    return;
    }

// Latch To Control 4
//-------------------
if(CheckBoundaries(&RectPlugInValue4))
    {
    ActiveControlType = AUTO_VALUE_4;
    ActiveControlValue = PlugInDataValue[FxActiveParamSet][AUTO_VALUE_4];
    ActiveControlValueMin = 0;
    ActiveControlValueMax = 100;
    ActiveControlValueDef = 0;
    ActiveControlMultiplier = 1;
    CopyRect(&ActiveControlRect, &RectPlugInValue4);
    
    ActiveControlListPtr = (LPSTR)NULL;

    TempLRCenter = ((ActiveControlRect.right - ActiveControlRect.left) / 2) + ActiveControlRect.left;

    if(CurMouseX > TempLRCenter)
        ActionControlScroll();
    else
        ActionControlMenuList();

    return;
    }

// Latch To Control 5
//-------------------
if(CheckBoundaries(&RectPlugInValue5))
    {
    ActiveControlType = AUTO_VALUE_5;
    ActiveControlValue = PlugInDataValue[FxActiveParamSet][AUTO_VALUE_5];
    ActiveControlValueMin = 0;
    ActiveControlValueMax = 100;
    ActiveControlValueDef = 0;
    ActiveControlMultiplier = 1;
    CopyRect(&ActiveControlRect, &RectPlugInValue5);
    
    ActiveControlListPtr = (LPSTR)NULL;

    TempLRCenter = ((ActiveControlRect.right - ActiveControlRect.left) / 2) + ActiveControlRect.left;

    if(CurMouseX > TempLRCenter)
        ActionControlScroll();
    else
        ActionControlMenuList();

    return;
    }

// Latch To Control 6
//-------------------
if(CheckBoundaries(&RectPlugInValue6))
    {
    ActiveControlType = AUTO_VALUE_6;
    ActiveControlValue = PlugInDataValue[FxActiveParamSet][AUTO_VALUE_6];
    ActiveControlValueMin = 0;
    ActiveControlValueMax = 100;
    ActiveControlValueDef = 0;
    ActiveControlMultiplier = 1;
    CopyRect(&ActiveControlRect, &RectPlugInValue6);
    
    ActiveControlListPtr = (LPSTR)NULL;

    TempLRCenter = ((ActiveControlRect.right - ActiveControlRect.left) / 2) + ActiveControlRect.left;

    if(CurMouseX > TempLRCenter)
        ActionControlScroll();
    else
        ActionControlMenuList();

    return;
    }

// Move Window
//------------
IQSMoveWindow(hWndPlugIn[FxActiveParamSet]);
SetFocus(hWndPlugIn[FxActiveParamSet]);
return;
}

//=========================================
// Check PlugIn Zones (Right Mouse Button)
//=========================================

DWORD   __stdcall   CheckPlugInZonesRight(void)

{
return(0);
}

//====================
// Draw Vertical Knob
//====================

void    __stdcall   DrawVertKnob(HDC hDC, LPRECT RectAddr, DWORD CenterPos, DWORD RefreshFlag)

{
RECT        TempRect;

// Draw Knob At Current Value Position
//------------------------------------
CopyRect(&TempRect, RectAddr);

if(RefreshFlag)
    RefreshBitmap(hDC, &TempRect, hPlugInMainBitmap);

SelectObject(hDC, hVertKnobPen);

TempRect.top = TempRect.top + CenterPos - 4;
TempRect.bottom = TempRect.top + 9;
TempRect.left = TempRect.left + 1;
FillRect(hDC, &TempRect, hBlackBrush);

TempRect.top = TempRect.top + 4;
MoveToEx(hDC, TempRect.left, TempRect.top, NULL);
LineTo(hDC, TempRect.right, TempRect.top);
return;
}

//=======================
// Lock To Vertical Knob
//=======================

void    __stdcall   LockToVertKnob(void)

{
POINT   pMousePos;
POINT   pClipZone;
RECT    TempRect;

CopyRect(&TempRect, &VertKnobRect);

// Set Mouse To Knob Center Position
//----------------------------------
pMousePos.x = TempRect.left + ((TempRect.right - TempRect.left) / 2);
pMousePos.y = TempRect.top + VertKnobPos;

OldMouseX = pMousePos.x;
OldMouseY = pMousePos.y;
ClientToScreen(VertKnobWindow, (LPPOINT) &pMousePos);
SetCursorPos(pMousePos.x, pMousePos.y);

// Set Clip Zone
//--------------
pClipZone.x = OldMouseX - 1;
pClipZone.y = TempRect.top + 5;
ClientToScreen(VertKnobWindow, &pClipZone);
TempRect.left = pClipZone.x;
TempRect.top = pClipZone.y;

pClipZone.x = OldMouseX + 1;
pClipZone.y = TempRect.bottom - 4;
ClientToScreen(VertKnobWindow, &pClipZone);
TempRect.right = pClipZone.x;
TempRect.bottom = pClipZone.y;

ClipCursor((LPRECT) &TempRect);
return;
}

//====================
// Move Vertical Knob
//====================

void    __stdcall   MoveVertKnob(void)

{
HDC     hDC;
HDC     TempMemoryDC;
RECT    TempRect;
DWORD   TempYNewPos;
DWORD   TempDir;
DWORD   TempChange;
DWORD   TempYStartPos;

if(CurMouseY == OldMouseY)
    return;

CopyRect(&TempRect, &VertKnobRect);

hDC = GetDC(VertKnobWindow);

// Calculate New Position
//-----------------------
if(CurMouseY > OldMouseY)
    {
    TempChange = CurMouseY - OldMouseY;
    TempYNewPos = VertKnobPos + TempChange;
    TempDir = 0;
    }
else
    {
    TempChange = OldMouseY - CurMouseY;
    TempYNewPos = VertKnobPos - TempChange;
    TempDir = 1;
    }

OldMouseY = CurMouseY;
OldMouseX = CurMouseX;

// Draw Knob In New Position
//--------------------------
DrawVertKnob(hDC, &VertKnobRect, TempYNewPos, 0);

// Draw Vertical Bitmap Over Old Knob Trail
//-----------------------------------------
TempMemoryDC = CreateCompatibleDC(hDC);
SelectObject(TempMemoryDC, VertKnobBackBitmap);

if(TempDir)
    {
    TempYStartPos = TempRect.top + TempYNewPos + 5;
    BitBlt(hDC, TempRect.left, TempYStartPos, TempRect.right - TempRect.left, TempChange + 1, TempMemoryDC, TempRect.left, TempYStartPos, SRCCOPY);
    }
else
    {
    TempYStartPos = TempRect.top + TempYNewPos - TempChange - 5;
    BitBlt(hDC, TempRect.left, TempYStartPos, TempRect.right - TempRect.left, TempChange + 1, TempMemoryDC, TempRect.left, TempYStartPos, SRCCOPY);
    }

DeleteDC(TempMemoryDC);
ReleaseDC(VertKnobWindow,hDC);

VertKnobPos = TempYNewPos;
return;
}

//=========================
// Set Vertical Knob Value
//=========================

void    __stdcall   SetVertKnobValue(void)

{
if(VertKnobCtrlId == AUTO_VOL_POT)
    {
    // Set New Value And Init Data
    //----------------------------
    PlugInDataValue[FxActiveParamSet][AUTO_VOL_POT] = VertKnobPos;
    InitPlugInVariables(FxActiveParamSet, AUTO_VOL_POT);

    // Send To Automation If Needed
    //-----------------------------
    SendAutomation(AUTO_VOL_POT, VertKnobPos, 5, 105);
    return;
    }

return;
}

//=================================
// Set Vertical Knob Default Value
//=================================

void    __stdcall   SetVertKnobValueDef(void)

{
// Skip If Knob Is Not In Active Mode
//-----------------------------------
if(VertKnobActive == 0)
    return;

if(VertKnobCtrlId == AUTO_VOL_POT)
    {
    // Set Default Value And Init Data
    //--------------------------------
    PlugInDataValue[FxActiveParamSet][AUTO_VOL_POT] = VertKnobPosDef;
    InitPlugInVariables(FxActiveParamSet, AUTO_VOL_POT);

    DrawVertKnob(PlugInDC[FxActiveParamSet], &RectPlugInVolPot, VertKnobPosDef, 1);

    // Send To Automation If Needed
    //-----------------------------
    SendAutomation(AUTO_VOL_POT, VertKnobPosDef, 5, 105);
    goto Finish;
    }

// Exit Knob Active Mode
//----------------------
Finish:
VertKnobActive = 0;
VertKnobCtrlId = NO_VALUE;
SendMessage(hWndSawMain, SawFxExecuteFunctionMsg, SET_EDL_MODIFY_FLAG, 0);
return;
}

//=================
// PlusMinus Check 
//=================

void    __stdcall   PlusMinusCheck(void)

{
POINT   pMousePos;

// Skip If Not In Active Mode
//---------------------------
if(PlusMinusActive == 0)
    return;

// Adjust Value If Needed
//-----------------------
GetCursorPos(&pMousePos);
ScreenToClient(hWndPlugIn[FxActiveParamSet], &pMousePos);

if(PlusMinusCenter == pMousePos.y)
    return;

CurMouseX = pMousePos.x;
CurMouseY = pMousePos.y;

// Call Active Control Function
//-----------------------------
(*ActionControlFunction)();
return;
}

//===============
// PlusMinus End 
//===============

void    __stdcall   PlusMinusEnd(void)

{
// End Plus-Minus Mode
//--------------------
ClipCursor(NULL);
SetCursor(LoadCursor(NULL, IDC_ARROW));

PlusMinusActive = 0;
FxBackThreadSuspendFlag = 1;
ActiveControlType = 0;

SendMessage(hWndSawMain, SawFxExecuteFunctionMsg, SET_EDL_MODIFY_FLAG, 0);
return;
}

//=======================
// Control Scroll Action
//=======================

void    __stdcall   ActionControlScroll(void)

{
POINT       pMousePos;
POINT       pClipZone;
int         TempOldMouseX;
int         TempOldMouseY;
RECT        TempRect;

// Click Only - Single Increment Or Decrement
//-------------------------------------------
if(WaitMsecForEvent(200, WM_LBUTTONUP))
    {
    if(CurMouseY < ((ActiveControlRect.bottom - ActiveControlRect.top) / 2) + ActiveControlRect.top)
        ActiveControlValue = ActiveControlValue + 1;
    else
        ActiveControlValue = ActiveControlValue - 1;

    if(ActiveControlValue < ActiveControlValueMin)
        ActiveControlValue = ActiveControlValueMin;

    if(ActiveControlValue > ActiveControlValueMax)
        ActiveControlValue = ActiveControlValueMax;

    // Update Control And Display
    //---------------------------
    ActionUpdateControl();

    ActiveControlType = 0;
    return;
    }

// Point To Ongoing Support Function
//----------------------------------
ActionControlFunction = (PROC)MoveControlScroll;

// Set Mouse To Right Center Of Zone
//----------------------------------
pMousePos.x = ActiveControlRect.right;
pMousePos.y = ((ActiveControlRect.bottom - ActiveControlRect.top) / 2) + ActiveControlRect.top;
TempOldMouseX = pMousePos.x;
TempOldMouseY = pMousePos.y;
ClientToScreen(hWndPlugIn[FxActiveParamSet], (LPPOINT) &pMousePos);

ShowCursor(FALSE);
SetCursorPos(pMousePos.x, pMousePos.y);

// Set Clip Zone
//--------------
pClipZone.x = TempOldMouseX;
pClipZone.y = ActiveControlRect.top;
ClientToScreen(hWndPlugIn[FxActiveParamSet], &pClipZone);
TempRect.left = pClipZone.x;
TempRect.top = pClipZone.y;

pClipZone.x = TempOldMouseX + 1;
pClipZone.y = (ActiveControlRect.bottom + 1);
ClientToScreen(hWndPlugIn[FxActiveParamSet], &pClipZone);
TempRect.right = pClipZone.x;
TempRect.bottom = pClipZone.y;

ClipCursor((LPRECT)&TempRect);

SetCursor(hPlusMinusCursor);
ShowCursor(TRUE);

// Activate Plus-Minus Mode And Resume Background Thread
//------------------------------------------------------
PlusMinusCenter = TempOldMouseY;
PlusMinusActive = 1;

FxBackThreadSuspendFlag = 0;
ResumeThread(hFxBackThread);
return;
}

//=====================
// Move Control Scroll
//=====================

void    __stdcall   MoveControlScroll(void)

{
DWORD       TempChange;

// Skip If Mouse Is On Center
//---------------------------
if(CurMouseY == PlusMinusCenter)
    return;

// Calculate New Value
//--------------------
if(CurMouseY > PlusMinusCenter)
    {
    TempChange = (CurMouseY - PlusMinusCenter) / 2;

    if(ChkVKey(VK_SHIFT))
        TempChange = TempChange * 2;

    ActiveControlValue = ActiveControlValue - TempChange;
    }
else
    {
    TempChange = (PlusMinusCenter - CurMouseY) / 2;

    if(ChkVKey(VK_SHIFT))
        TempChange = TempChange * 2;

    ActiveControlValue = ActiveControlValue + TempChange;
    }

if(TempChange == 0)
    return;

if(TempChange <= 2)
    Sleep(80);

if(ActiveControlValue < ActiveControlValueMin)
    ActiveControlValue = ActiveControlValueMin;

if(ActiveControlValue > ActiveControlValueMax)
    ActiveControlValue = ActiveControlValueMax;

// Update Control And Display
//---------------------------
ActionUpdateControl();
return;
}

//========================
// Popup Menu List Action
//========================

void    __stdcall   ActionControlMenuList(void)

{
int         TempXStart;
int         TempYStart;
POINT       TempPt;
int         TempID;
int         TempCount;
DWORD       TempBarBreak;
int         TempRetValue;
DWORD       TempSkip;
int         TempCurValue;

GetCursorPos(&TempPt);
TempXStart = TempPt.x;
TempYStart = TempPt.y;

TempSkip = 0;

SendMessage(hWndSawMain, SawFxExecuteFunctionMsg, SET_MAX_READ_AHEAD, 0);

// Create Menu
//------------
hMenuList = CreatePopupMenu();

if(ActiveControlListPtr == NULL)
    {
    // Fill In List Of Incrementing Values
    //------------------------------------
    TempID = 0;
    TempBarBreak = 0;

    TempCurValue = ActiveControlValue;

    for(TempCount = ActiveControlValueMin / ActiveControlMultiplier; TempCount <= ActiveControlValueMax / ActiveControlMultiplier; TempCount++)
        {
        wsprintf(szWorkBuff, "%u", TempCount);

        if(TempBarBreak >= 30)
            {
            AppendMenu(hMenuList, MFT_STRING | MFT_MENUBARBREAK, IDM_MENULIST_BASE + TempID, (LPCTSTR)szWorkBuff);
            TempBarBreak = 1;
            }
        else
            {
            AppendMenu(hMenuList, MFT_STRING, IDM_MENULIST_BASE + TempID, (LPCTSTR)szWorkBuff);
            TempBarBreak = TempBarBreak + 1;
            }

        if((TempCount * ActiveControlMultiplier) == ActiveControlValueDef)
            SetMenuDefaultItem(hMenuList, IDM_MENULIST_BASE + TempID, FALSE);

        if((TempCount * ActiveControlMultiplier) == TempCurValue)
            CheckMenuItem(hMenuList, IDM_MENULIST_BASE + TempID, MF_BYCOMMAND | MF_CHECKED);

        TempID++;
        }
    }
else
    {
    // Fill In List Of Values From String List
    //----------------------------------------
    if(ActiveControlValueMax > 300)
        TempSkip = 1;

    TempID = 0;
    TempBarBreak = 0;

    for(TempCount = ActiveControlValueMin; TempCount <= ActiveControlValueMax; TempCount++)
        {
        lstrcpy(szWorkBuff, ActiveControlListPtr + (MAX_LIST_CHARS * TempCount));

        if(TempBarBreak >= 30)
            {
            AppendMenu(hMenuList, MFT_STRING | MFT_MENUBARBREAK, IDM_MENULIST_BASE + TempID, (LPCTSTR)szWorkBuff);
            TempBarBreak = 1;
            }
        else
            {
            AppendMenu(hMenuList, MFT_STRING, IDM_MENULIST_BASE + TempID, (LPCTSTR)szWorkBuff);
            TempBarBreak = TempBarBreak + 1;
            }

        if(TempCount == ActiveControlValue)
            CheckMenuItem(hMenuList, IDM_MENULIST_BASE + TempID, MF_BYCOMMAND | MF_CHECKED);

        TempID = TempID + 1;

        if(TempSkip)
            {
            TempCount++;
            TempID = TempID + 1;
            }
        }

    // Set DefaultValue
    //-----------------
    SetMenuDefaultItem(hMenuList, ActiveControlValueDef - ActiveControlValueMin, TRUE);
    }

// Display Menu
//-------------
TempRetValue = TrackPopupMenu(hMenuList, TPM_LEFTALIGN | TPM_RIGHTBUTTON | TPM_NONOTIFY | TPM_RETURNCMD, TempXStart, TempYStart, 0, hWndPlugIn[FxActiveParamSet], NULL); 

DestroyMenu(hMenuList);
ClearAllMouseMsgs();

SendMessage(hWndSawMain, SawFxExecuteFunctionMsg, SET_MIN_READ_AHEAD, 0);

if(TempRetValue == 0)
    return;

TempRetValue = TempRetValue + (ActiveControlValueMin / ActiveControlMultiplier) - IDM_MENULIST_BASE;

if(TempRetValue == ActiveControlValue)
    return;

ActiveControlValue = TempRetValue * ActiveControlMultiplier;

// Update Control And Display
//---------------------------
ActionUpdateControl();
return;
}

//=============================
// Action Set To Default Value 
//=============================

void    __stdcall   ActionSetToDefaultValue(void)

{
// Clear Mode
//-----------
ClipCursor(NULL);
SetCursor(LoadCursor(NULL, IDC_ARROW));

// Set To Default Value
//---------------------
ActiveControlValue = ActiveControlValueDef;

// Update Control And Display
//---------------------------
ActionUpdateControl();
return;
}

//==================================
// Update Control Value And Display
//==================================

void    __stdcall   ActionUpdateControl(void)

{
// Set New Value And Initialize
// Send To Automation If Needed
// Display New Value
//----------------------------- 
if(ActiveControlType == AUTO_VALUE_1)
    {
    PlugInDataValue[FxActiveParamSet][AUTO_VALUE_1] = ActiveControlValue;
    InitPlugInVariables(FxActiveParamSet, AUTO_VALUE_1);

    SendAutomation(AUTO_VALUE_1, ActiveControlValue, 0, 100);

    DisplayPlugInValue(PlugInDC[FxActiveParamSet], AUTO_VALUE_1, ActiveControlValue, 1);
    return;
    }

if(ActiveControlType == AUTO_VALUE_2)
    {
    PlugInDataValue[FxActiveParamSet][AUTO_VALUE_2] = ActiveControlValue;
    InitPlugInVariables(FxActiveParamSet, AUTO_VALUE_2);

    SendAutomation(AUTO_VALUE_2, ActiveControlValue, 0, 100);

    DisplayPlugInValue(PlugInDC[FxActiveParamSet], AUTO_VALUE_2, ActiveControlValue, 1);
    return;
    }

if(ActiveControlType == AUTO_VALUE_3)
    {
    PlugInDataValue[FxActiveParamSet][AUTO_VALUE_3] = ActiveControlValue;
    InitPlugInVariables(FxActiveParamSet, AUTO_VALUE_3);

    SendAutomation(AUTO_VALUE_3, ActiveControlValue, 0, 100);

    DisplayPlugInValue(PlugInDC[FxActiveParamSet], AUTO_VALUE_3, ActiveControlValue, 1);
    return;
    }

if(ActiveControlType == AUTO_VALUE_4)
    {
    PlugInDataValue[FxActiveParamSet][AUTO_VALUE_4] = ActiveControlValue;
    InitPlugInVariables(FxActiveParamSet, AUTO_VALUE_4);

    SendAutomation(AUTO_VALUE_4, ActiveControlValue, 0, 100);

    DisplayPlugInValue(PlugInDC[FxActiveParamSet], AUTO_VALUE_4, ActiveControlValue, 1);
    return;
    }

if(ActiveControlType == AUTO_VALUE_5)
    {
    PlugInDataValue[FxActiveParamSet][AUTO_VALUE_5] = ActiveControlValue;
    InitPlugInVariables(FxActiveParamSet, AUTO_VALUE_5);

    SendAutomation(AUTO_VALUE_5, ActiveControlValue, 0, 100);

    DisplayPlugInValue(PlugInDC[FxActiveParamSet], AUTO_VALUE_5, ActiveControlValue, 1);
    return;
    }

if(ActiveControlType == AUTO_VALUE_6)
    {
    PlugInDataValue[FxActiveParamSet][AUTO_VALUE_6] = ActiveControlValue;
    InitPlugInVariables(FxActiveParamSet, AUTO_VALUE_6);

    SendAutomation(AUTO_VALUE_6, ActiveControlValue, 0, 100);

    DisplayPlugInValue(PlugInDC[FxActiveParamSet], AUTO_VALUE_6, ActiveControlValue, 1);
    return;
    }

return;
}

//============================
// Create Info Selection Menu
//============================

void    __stdcall   CreateOptionsSelectionMenu(void)

{
int     TempXStart;
int     TempYStart;
POINT   TempPt;
int     TempRetValue;

GetCursorPos(&TempPt);
TempXStart = TempPt.x + 4;
TempYStart = TempPt.y + 4;

// Create Menu
//------------
hMenuOptions = CreatePopupMenu();

// Fill In List Of Options
//------------------------
AppendMenu(hMenuOptions, MF_STRING, IDM_OPTIONS_LOAD_PRESETS, (LPCTSTR)"Load Presets");
AppendMenu(hMenuOptions, MF_STRING, IDM_OPTIONS_SAVE_PRESETS, (LPCTSTR)"Save Presets");

AppendMenu(hMenuOptions, MF_SEPARATOR, 0, NULL);
AppendMenu(hMenuOptions, MF_STRING, IDM_OPTIONS_ABOUT, (LPCTSTR)"About");

// Display Menu
//-------------
TempRetValue = TrackPopupMenu(hMenuOptions, TPM_LEFTALIGN | TPM_LEFTBUTTON | TPM_NONOTIFY | TPM_RETURNCMD, TempXStart, TempYStart, 0, hWndPlugIn[FxActiveParamSet], NULL);

DestroyMenu(hMenuOptions);
ClearAllMouseMsgs();

if(TempRetValue == 0)
    return;

// Execute Proper Function
//------------------------
if(TempRetValue == IDM_OPTIONS_LOAD_PRESETS)
    {
    LoadPlugInPreset();
    return;
    }

if(TempRetValue == IDM_OPTIONS_SAVE_PRESETS)
    {
    SavePlugInPreset();
    return;
    }

if(TempRetValue == IDM_OPTIONS_ABOUT)
    {
    CreateAboutWindow();    
    return;
    }

return;
}

//=================
// Send Automation
//=================

void    __stdcall   SendAutomation(DWORD TempCtrlId, int TempDataValue, int TempMinDataValue, int TempMaxDataValue)

{
// Set As New Default Value If Automation Mode Is Not Active
//----------------------------------------------------------
if(FxPtrSawData->FX_MTAutomationWriteFlag == 0)
    {
    PlugInDefaultDataValue[FxActiveParamSet][TempCtrlId] = TempDataValue;
    return;
    }

// Send To Automation
//-------------------
FxPtrSawData->FX_Function_DWord_Param1 = FxPlugInIndex;
FxPtrSawData->FX_Function_DWord_Param2 = FxActiveParamSet;
FxPtrSawData->FX_Function_DWord_Param3 = PlugInTrack[FxActiveParamSet];
FxPtrSawData->FX_Function_DWord_Param4 = TempCtrlId;
FxPtrSawData->FX_Function_Int_Param1 = TempDataValue;
FxPtrSawData->FX_Function_Int_Param2 = TempMinDataValue;
FxPtrSawData->FX_Function_Int_Param3 = TempMaxDataValue;

SendMessage(hWndSawMain, SawFxExecuteFunctionMsg, SEND_TO_AUTOMATION, 0);
return;
}

//====================
// Save PlugIn Preset
//====================

void    __stdcall   SavePlugInPreset(void)

{
BOOL    Result;
int     TempFileHandle;
LPBYTE  TempDataPtr;
DWORD   TempDataSet;
DWORD   TempDataBuffSize;

// Select FileName
//----------------
lstrcpy((LPSTR)szStringBuff, "Save PlugIn Preset");
Result = PickSaveFileName(LastPlugInFileName, szDefaultPath, szAllFilters, PLG_FILE, szPlgQual, 1);

// Canceled
//---------
if(Result == 0)
    return;

// Save New Settings
//------------------
IQSGetCurDirectory(szDefaultPath);
lstrcpy(LastPlugInFileName, szSaveName + ofFileName.nFileOffset);

lstrcpy(szWork1Name, szSaveName);

// Create New File Or OverWrite Old One
//-------------------------------------
TempFileHandle = OpenFile(szWork1Name, &ofFileStructure, OF_CREATE);

if(TempFileHandle == HFILE_ERROR)
    return;

// Organize Data In Memory Buffer
//-------------------------------
TempDataPtr = PtrBuffWork1;
TempDataSet = FxActiveParamSet;
TempDataBuffSize = 0;

// Save Header
//------------
lstrcpy((LPSTR)TempDataPtr, "FX API TEST PLUGIN PRESET 001  ");
TempDataBuffSize = TempDataBuffSize + 32;

// Save PlugIn Current Data
//-------------------------
*(PDWORD)(TempDataPtr + TempDataBuffSize) = PlugInDataValue[TempDataSet][AUTO_VOL_POT];
TempDataBuffSize = TempDataBuffSize + 4;

*(PDWORD)(TempDataPtr + TempDataBuffSize) = PlugInDataValue[TempDataSet][AUTO_VALUE_1];
TempDataBuffSize = TempDataBuffSize + 4;

*(PDWORD)(TempDataPtr + TempDataBuffSize) = PlugInDataValue[TempDataSet][AUTO_VALUE_2];
TempDataBuffSize = TempDataBuffSize + 4;

*(PDWORD)(TempDataPtr + TempDataBuffSize) = PlugInDataValue[TempDataSet][AUTO_VALUE_3];
TempDataBuffSize = TempDataBuffSize + 4;

*(PDWORD)(TempDataPtr + TempDataBuffSize) = PlugInDataValue[TempDataSet][AUTO_VALUE_4];
TempDataBuffSize = TempDataBuffSize + 4;

*(PDWORD)(TempDataPtr + TempDataBuffSize) = PlugInDataValue[TempDataSet][AUTO_VALUE_5];
TempDataBuffSize = TempDataBuffSize + 4;

*(PDWORD)(TempDataPtr + TempDataBuffSize) = PlugInDataValue[TempDataSet][AUTO_VALUE_6];
TempDataBuffSize = TempDataBuffSize + 4;

// Write Data To Disk
//-------------------
_lwrite(TempFileHandle, (LPSTR)TempDataPtr, TempDataBuffSize);
_lclose(TempFileHandle);
return;
}

//====================
// Load PlugIn Preset
//====================

void    __stdcall   LoadPlugInPreset(void)

{
DWORD   TempByteCount;
int     TempFileHandle;
BOOL    Result;
LPBYTE  TempDataPtr;
DWORD   TempDataSet;

// Select FileName
//----------------
lstrcpy((LPSTR)szStringBuff, "Load PlugIn Preset");
Result = PickOpenFileName(LastPlugInFileName, szDefaultPath, szAllFilters, PLG_FILE, szPlgQual);

// Canceled
//---------
if(Result == 0)
    return;

// Save New Settings
//------------------
IQSGetCurDirectory(szDefaultPath);
lstrcpy(LastPlugInFileName, szOpenName + ofFileName.nFileOffset);

lstrcpy(szWork1Name, szOpenName);

// Read File Into Memory
//----------------------
TempFileHandle = OpenFile(szWork1Name, &ofFileStructure, OF_READ);

if(TempFileHandle == HFILE_ERROR)
    return;

TempDataPtr = PtrBuffWork1;
TempDataSet = FxActiveParamSet;
    
// Ask For More Data Than The File Size Will Ever Be
//--------------------------------------------------
_lread(TempFileHandle, TempDataPtr, 0x0000f000);
_lclose(TempFileHandle);

// Check For Proper File Format
//-----------------------------
if(lstrcmp((LPSTR)TempDataPtr, "FX API TEST PLUGIN PRESET 001  ") != 0)
    {
    MessageBox(NULL, ImproperPresetMsg, WarningMsg, MB_SYSTEMMODAL | MB_ICONEXCLAMATION | MB_OK);
    return;
    }

// Set Default Variables From Memory Data
//---------------------------------------
TempByteCount = 32;

PlugInDefaultDataValue[TempDataSet][AUTO_VOL_POT] = *(PDWORD)(TempDataPtr + TempByteCount);
TempByteCount = TempByteCount + 4;

PlugInDefaultDataValue[TempDataSet][AUTO_VALUE_1] = *(PDWORD)(TempDataPtr + TempByteCount);
TempByteCount = TempByteCount + 4;

PlugInDefaultDataValue[TempDataSet][AUTO_VALUE_2] = *(PDWORD)(TempDataPtr + TempByteCount);
TempByteCount = TempByteCount + 4;

PlugInDefaultDataValue[TempDataSet][AUTO_VALUE_3] = *(PDWORD)(TempDataPtr + TempByteCount);
TempByteCount = TempByteCount + 4;

PlugInDefaultDataValue[TempDataSet][AUTO_VALUE_4] = *(PDWORD)(TempDataPtr + TempByteCount);
TempByteCount = TempByteCount + 4;

PlugInDefaultDataValue[TempDataSet][AUTO_VALUE_5] = *(PDWORD)(TempDataPtr + TempByteCount);
TempByteCount = TempByteCount + 4;

PlugInDefaultDataValue[TempDataSet][AUTO_VALUE_6] = *(PDWORD)(TempDataPtr + TempByteCount);
TempByteCount = TempByteCount + 4;

// Set All Current Values To The New Defaults
//-------------------------------------------
CopyMemory(&PlugInDataValue[TempDataSet][0], &PlugInDefaultDataValue[TempDataSet][0], MAX_AUTO_CONTROLS * 4);

// Init PlugIn Variables
//----------------------
InitPlugInVariables(TempDataSet, FX_ALL_CONTROLS);

// Update The Display If Needed
//-----------------------------
if(hWndPlugIn[TempDataSet])
    {
    InvalidateRect(hWndPlugIn[TempDataSet], NULL, FALSE);
    UpdateWindow(hWndPlugIn[TempDataSet]);
    }
    
return;
}

//=======================
// Init PlugIn Variables
//=======================

void    __stdcall   InitPlugInVariables(DWORD TempParamSet, DWORD TempControlID)

{
DWORD   TempIndex;

// Init All Controls For This Parameter Set
//-----------------------------------------
if(TempControlID == FX_ALL_CONTROLS)
    {
    // Clear Meters If Plug-In Is Bypassed During RealTime Play
    //---------------------------------------------------------
    if(FxProcessFlag == REALTIME_PLAYBACK)
        {
        if(PlugInDataValue[TempParamSet][AUTO_BYPASS_SW])
            ResetMeterDisplay(TempParamSet);
        }

    // Calculate Volume Factor
    //------------------------
    FxVolFactor[TempParamSet] = 105 - PlugInDataValue[TempParamSet][AUTO_VOL_POT];

    // Set Output Format
    //------------------
    FxOutputFormat[TempParamSet] = PlugInDataValue[TempParamSet][AUTO_VALUE_1];

    // Set Meter Decay Value
    //----------------------
    TempIndex = PlugInDataValue[TempParamSet][AUTO_VALUE_2];
    PlugInMeterDecayFactor[TempParamSet] = MeterResponseValue[TempIndex];
    return;
    }

// Init Specified Control Only For This Parameter Set
//---------------------------------------------------
if(TempControlID == AUTO_BYPASS_SW)
    {
    // Clear Meters If Plug-In Is Bypassed During RealTime Play
    //---------------------------------------------------------
    if(FxProcessFlag == REALTIME_PLAYBACK)
        {
        if(PlugInDataValue[TempParamSet][AUTO_BYPASS_SW])
            ResetMeterDisplay(TempParamSet);
        }
    }

if(TempControlID == AUTO_VOL_POT)
    {
    // Calculate Volume Factor
    //------------------------
    FxVolFactor[TempParamSet] = 105 - PlugInDataValue[TempParamSet][AUTO_VOL_POT];
    return;
    }

if(TempControlID == AUTO_VALUE_1)
    {
    // Set Output Format
    //------------------
    FxOutputFormat[TempParamSet] = PlugInDataValue[TempParamSet][AUTO_VALUE_1];
    return;
    }

if(TempControlID == AUTO_VALUE_2)
    {
    // Set Meter Decay Value
    //----------------------
    TempIndex = PlugInDataValue[TempParamSet][AUTO_VALUE_2];
    PlugInMeterDecayFactor[TempParamSet] = MeterResponseValue[TempIndex];
    return;
    }

return;
}

//===================
// Init PlugIn Meter
//===================

void    __stdcall   InitPlugInMeter(DWORD TempParamSet)

{
// Init Meter Variables
//---------------------
PlugInMeterIndexHead[TempParamSet] = 0;
PlugInMeterIndexTail[TempParamSet] = 0;

PlugInMeterLastValueL[TempParamSet] = 0;
PlugInMeterLastValueR[TempParamSet] = 0;

PlugInMeterPos[TempParamSet][0] = END_OF_LIST;
PlugInMeterValueL[TempParamSet][0] = 0;
PlugInMeterValueR[TempParamSet][0] = 0;
return;
}

//============================
// Reset PlugIn Parameter Set
//============================

void    __stdcall   ResetParamSet(DWORD TempParamSet)

{
DWORD   TempSet;
DWORD   TempStartSet;
DWORD   TempEndSet;

// Set Start And End Indexes (Might Be All Sets)
//----------------------------------------------
if(TempParamSet == FX_ALL_PARAM_SETS)
    {
    TempStartSet = 0;
    TempEndSet = MAX_PARAM_SETS;

    PlugInParamSetCount = 0;
    }
else
    {
    TempStartSet = TempParamSet;    
    TempEndSet = TempStartSet + 1;    

    if(PlugInParamSetCount)
        PlugInParamSetCount = PlugInParamSetCount - 1;
    }
    
// Loop And Reset All Variables For Each Parameter Set
//----------------------------------------------------
for(TempSet = TempStartSet; TempSet < TempEndSet; TempSet++)
    {
    PlugInParamSetTable[TempSet] = 0;
    PlugInTrack[TempSet] = NO_ASSIGNMENT;
    PlugInBytesPerSample[TempSet] = 4;

    PlugInDefaultDataValue[TempSet][AUTO_BYPASS_SW] = 0;
    PlugInDataValue[TempSet][AUTO_BYPASS_SW] = 0;

    PlugInDefaultDataValue[TempSet][AUTO_VOL_POT] = 5;
    PlugInDataValue[TempSet][AUTO_VOL_POT] = 5;

    PlugInDefaultDataValue[TempSet][AUTO_VALUE_1] = 0;
    PlugInDataValue[TempSet][AUTO_VALUE_1] = 0;

    PlugInDefaultDataValue[TempSet][AUTO_VALUE_2] = 0;
    PlugInDataValue[TempSet][AUTO_VALUE_2] = 0;

    PlugInDefaultDataValue[TempSet][AUTO_VALUE_3] = 0;
    PlugInDataValue[TempSet][AUTO_VALUE_3] = 0;

    PlugInDefaultDataValue[TempSet][AUTO_VALUE_4] = 0;
    PlugInDataValue[TempSet][AUTO_VALUE_4] = 0;

    PlugInDefaultDataValue[TempSet][AUTO_VALUE_5] = 0;
    PlugInDataValue[TempSet][AUTO_VALUE_5] = 0;

    PlugInDefaultDataValue[TempSet][AUTO_VALUE_6] = 0;
    PlugInDataValue[TempSet][AUTO_VALUE_6] = 0;
    }

return;
}

//=================================
// Do PlugIn Process 32 Bit Stereo
//
// 4 Bytes Per Sample Per Channel
//=================================

void    __stdcall   DoPlugInProcess32(void)

{
int     TempBuffSize;
int     TempValue;
int     TempValueL;
int     TempValueR;
int     TempPeakValueL;
int     TempPeakValueR;
DWORD   TempIndex;
LPBYTE  TempDataPtr;

// Skip If There Is Nothing To Process
//------------------------------------
if(FxProcessBuffSize == 0)
    return;

// Do Volume Process
//------------------
TempBuffSize = FxProcessBuffSize;
TempDataPtr = FxProcessBuffPtr;

while(TRUE)
    {
    // Adjust Value Of Each Sample
    //----------------------------
    TempValue = *(PINT)(TempDataPtr);
    TempValue = (TempValue * FxVolFactor[FxProcessParamSet]) / 100;
    *(PINT)(TempDataPtr) = TempValue;

    TempValue = *(PINT)(TempDataPtr + 4);
    TempValue = (TempValue * FxVolFactor[FxProcessParamSet]) / 100;
    *(PINT)(TempDataPtr + 4) = TempValue;

    TempDataPtr = TempDataPtr + 8;
    TempBuffSize = TempBuffSize - 8;

    if(TempBuffSize <= 0)
        break;
    }

// Do Mono Format
//---------------
if(FxOutputFormat[FxProcessParamSet] == 1)
    {
    TempBuffSize = FxProcessBuffSize;
    TempDataPtr = FxProcessBuffPtr;

    while(TRUE)
        {
        // Adjust Left Value Down 6db
        //---------------------------
        TempValueL = *(PINT)(TempDataPtr);
        TempValueL = TempValueL / 2;
        
        // Adjust Right Value Down 6db
        //----------------------------
        TempValueR = *(PINT)(TempDataPtr + 4);
        TempValueR = TempValueR / 2;

        // Add Them Together And Store The Result
        //---------------------------------------
        TempValue = TempValueL + TempValueR;
        *(PINT)(TempDataPtr) = TempValue;
        *(PINT)(TempDataPtr + 4) = TempValue;

        TempDataPtr = TempDataPtr + 8;
        TempBuffSize = TempBuffSize - 8;

        if(TempBuffSize <= 0)
            break;
        }
    }

// Do Lft Only Format
//-------------------
if(FxOutputFormat[FxProcessParamSet] == 2)
    {
    TempBuffSize = FxProcessBuffSize;
    TempDataPtr = FxProcessBuffPtr;

    while(TRUE)
        {
        // Clear All Right Channel Samples
        //--------------------------------
        *(PINT)(TempDataPtr + 4) = 0;

        TempDataPtr = TempDataPtr + 8;
        TempBuffSize = TempBuffSize - 8;

        if(TempBuffSize <= 0)
            break;
        }
    }

// Do Rgt Only Format
//-------------------
if(FxOutputFormat[FxProcessParamSet] == 3)
    {
    TempBuffSize = FxProcessBuffSize;
    TempDataPtr = FxProcessBuffPtr;

    while(TRUE)
        {
        // Clear All Left Channel Samples
        //-------------------------------
        *(PINT)(TempDataPtr) = 0;

        TempDataPtr = TempDataPtr + 8;
        TempBuffSize = TempBuffSize - 8;

        if(TempBuffSize <= 0)
            break;
        }
    }

// Do Lft Mono Format
//-------------------
if(FxOutputFormat[FxProcessParamSet] == 4)
    {
    TempBuffSize = FxProcessBuffSize;
    TempDataPtr = FxProcessBuffPtr;

    while(TRUE)
        {
        // Store Left Value To Right Channel
        //----------------------------------
        TempValue = *(PINT)(TempDataPtr);
        *(PINT)(TempDataPtr + 4) = TempValue;

        TempDataPtr = TempDataPtr + 8;
        TempBuffSize = TempBuffSize - 8;

        if(TempBuffSize <= 0)
            break;
        }
    }

// Do Rgt Mono Format
//-------------------
if(FxOutputFormat[FxProcessParamSet] == 5)
    {
    TempBuffSize = FxProcessBuffSize;
    TempDataPtr = FxProcessBuffPtr;

    while(TRUE)
        {
        // Store Right Value To Left Channel
        //----------------------------------
        TempValue = *(PINT)(TempDataPtr + 4);
        *(PINT)(TempDataPtr) = TempValue;

        TempDataPtr = TempDataPtr + 8;
        TempBuffSize = TempBuffSize - 8;

        if(TempBuffSize <= 0)
            break;
        }
    }

// Collect Meter Peak Data For Synchronized Display
//-------------------------------------------------
TempBuffSize = FxProcessBuffSize;
TempDataPtr = FxProcessBuffPtr;
TempPeakValueL = 0;
TempPeakValueR = 0;

while(TRUE)
    {
    // Get Left Value As Absolute Positive Value
    //------------------------------------------
    TempValue = *(PINT)(TempDataPtr);

    if(TempValue < 0)
        TempValue = 0 - TempValue;

    // Store As The Highest Peak Value If Needed
    //------------------------------------------
    if(TempValue > TempPeakValueL)
        TempPeakValueL = TempValue;

    // Get Right Value As Absolute Positive Value
    //-------------------------------------------
    TempValue = *(PINT)(TempDataPtr + 4);

    if(TempValue < 0)
        TempValue = 0 - TempValue;

    // Store As The Highest Peak Value If Needed
    //------------------------------------------
    if(TempValue > TempPeakValueR)
        TempPeakValueR = TempValue;

    TempDataPtr = TempDataPtr + 8;
    TempBuffSize = TempBuffSize - 8;

    // At End Of Buffer Store Final Results For Later Display
    //-------------------------------------------------------
    if(TempBuffSize <= 0)
        {
        // Trim Value To Max Clip Value If Needed
        //---------------------------------------
        if(TempPeakValueL > 0x007fffff)
            TempPeakValueL = 0x007fffff;

        if(TempPeakValueR > 0x007fffff)
            TempPeakValueR = 0x007fffff;

        // Scale The Peak Value To A Meter Segment Value
        //----------------------------------------------
        for(TempIndex = 0; TempIndex < MAX_METER_SEGMENTS; TempIndex++)
            {
            if(TempPeakValueL <= MeterTable[TempIndex])
                {
                if(TempIndex > 0)
                    TempIndex--;

                TempPeakValueL = TempIndex;
                break;
                }
            }

        for(TempIndex = 0; TempIndex < MAX_METER_SEGMENTS; TempIndex++)
            {
            if(TempPeakValueR <= MeterTable[TempIndex])
                {
                if(TempIndex > 0)
                    TempIndex--;

                TempPeakValueR = TempIndex;
                break;
                }
            }

        // Simulate Analog Decay For Values Smaller Than The Last Values
        //--------------------------------------------------------------
        if(TempPeakValueL < PlugInMeterLastValueL[FxProcessParamSet] - PlugInMeterDecayFactor[FxProcessParamSet])
            TempPeakValueL = PlugInMeterLastValueL[FxProcessParamSet] - PlugInMeterDecayFactor[FxProcessParamSet];
                
        if(TempPeakValueR < PlugInMeterLastValueR[FxProcessParamSet] - PlugInMeterDecayFactor[FxProcessParamSet])
            TempPeakValueR = PlugInMeterLastValueR[FxProcessParamSet] - PlugInMeterDecayFactor[FxProcessParamSet];

        // Save These Values For Next Time
        //--------------------------------
        PlugInMeterLastValueL[FxProcessParamSet] = TempPeakValueL;
        PlugInMeterLastValueR[FxProcessParamSet] = TempPeakValueR;

        // Store The Results At The Index Head Position In The Meter Array
        //----------------------------------------------------------------
        TempIndex = PlugInMeterIndexHead[FxProcessParamSet];

        PlugInMeterPos[FxProcessParamSet][TempIndex] = FxProcessPos;
        PlugInMeterValueL[FxProcessParamSet][TempIndex] = TempPeakValueL;
        PlugInMeterValueR[FxProcessParamSet][TempIndex] = TempPeakValueR;

        // Increment Index Head Counter And Set Next Array Value To End Of List Value
        //---------------------------------------------------------------------------
        TempIndex++;

        if(TempIndex >= MAX_METER_INDEXES)
            TempIndex = 0;

        PlugInMeterIndexHead[FxProcessParamSet] = TempIndex;

        PlugInMeterPos[FxProcessParamSet][TempIndex] = END_OF_LIST;
        break;
        }
    }

return;
}

//=================================
// Do PlugIn Process 16 Bit Stereo
//
// 2 Bytes Per Sample Per Channel
//=================================

void    __stdcall   DoPlugInProcess16(void)

{
int     TempBuffSize;
int     TempValue;
int     TempValueL;
int     TempValueR;
int     TempPeakValueL;
int     TempPeakValueR;
DWORD   TempIndex;
LPBYTE  TempDataPtr;

// Skip If There Is Nothing To Process
//------------------------------------
if(FxProcessBuffSize == 0)
    return;

// Do Volume Process
//------------------
TempBuffSize = FxProcessBuffSize;
TempDataPtr = FxProcessBuffPtr;

while(TRUE)
    {
    // Adjust Value Of Each Sample
    //----------------------------
    TempValue = (int)*(PSHORT)(TempDataPtr);
    TempValue = (TempValue * FxVolFactor[FxProcessParamSet]) / 100;
    *(PSHORT)(TempDataPtr) = (short)TempValue;

    TempValue = (int)*(PSHORT)(TempDataPtr + 2);
    TempValue = (TempValue * FxVolFactor[FxProcessParamSet]) / 100;
    *(PSHORT)(TempDataPtr + 2) = (short)TempValue;

    TempDataPtr = TempDataPtr + 4;
    TempBuffSize = TempBuffSize - 4;

    if(TempBuffSize <= 0)
        break;
    }

// Do Mono Format
//---------------
if(FxOutputFormat[FxProcessParamSet] == 1)
    {
    TempBuffSize = FxProcessBuffSize;
    TempDataPtr = FxProcessBuffPtr;

    while(TRUE)
        {
        // Adjust Left Value Down 6db
        //---------------------------
        TempValueL = (int)*(PSHORT)(TempDataPtr);
        TempValueL = TempValueL / 2;
        
        // Adjust Right Value Down 6db
        //----------------------------
        TempValueR = (int)*(PSHORT)(TempDataPtr + 2);
        TempValueR = TempValueR / 2;

        // Add Them Together And Store The Result
        //---------------------------------------
        TempValue = TempValueL + TempValueR;
        *(PSHORT)(TempDataPtr) = (short)TempValue;
        *(PSHORT)(TempDataPtr + 2) = (short)TempValue;

        TempDataPtr = TempDataPtr + 4;
        TempBuffSize = TempBuffSize - 4;

        if(TempBuffSize <= 0)
            break;
        }
    }

// Do Lft Only Format
//-------------------
if(FxOutputFormat[FxProcessParamSet] == 2)
    {
    TempBuffSize = FxProcessBuffSize;
    TempDataPtr = FxProcessBuffPtr;

    while(TRUE)
        {
        // Clear All Right Channel Samples
        //--------------------------------
        *(PSHORT)(TempDataPtr + 2) = 0;

        TempDataPtr = TempDataPtr + 4;
        TempBuffSize = TempBuffSize - 4;

        if(TempBuffSize <= 0)
            break;
        }
    }

// Do Rgt Only Format
//-------------------
if(FxOutputFormat[FxProcessParamSet] == 3)
    {
    TempBuffSize = FxProcessBuffSize;
    TempDataPtr = FxProcessBuffPtr;

    while(TRUE)
        {
        // Clear All Left Channel Samples
        //-------------------------------
        *(PSHORT)(TempDataPtr) = 0;

        TempDataPtr = TempDataPtr + 4;
        TempBuffSize = TempBuffSize - 4;

        if(TempBuffSize <= 0)
            break;
        }
    }

// Do Lft Mono Format
//-------------------
if(FxOutputFormat[FxProcessParamSet] == 4)
    {
    TempBuffSize = FxProcessBuffSize;
    TempDataPtr = FxProcessBuffPtr;

    while(TRUE)
        {
        // Store Left Value To Right Channel
        //----------------------------------
        TempValue = (int)*(PSHORT)(TempDataPtr);
        *(PSHORT)(TempDataPtr + 2) = (short)TempValue;

        TempDataPtr = TempDataPtr + 4;
        TempBuffSize = TempBuffSize - 4;

        if(TempBuffSize <= 0)
            break;
        }
    }

// Do Rgt Mono Format
//-------------------
if(FxOutputFormat[FxProcessParamSet] == 5)
    {
    TempBuffSize = FxProcessBuffSize;
    TempDataPtr = FxProcessBuffPtr;

    while(TRUE)
        {
        // Store Right Value To Left Channel
        //----------------------------------
        TempValue = (int)*(PSHORT)(TempDataPtr + 2);
        *(PSHORT)(TempDataPtr) = (short)TempValue;

        TempDataPtr = TempDataPtr + 4;
        TempBuffSize = TempBuffSize - 4;

        if(TempBuffSize <= 0)
            break;
        }
    }

// Collect Meter Peak Data For Synchronized Display
//-------------------------------------------------
TempBuffSize = FxProcessBuffSize;
TempDataPtr = FxProcessBuffPtr;
TempPeakValueL = 0;
TempPeakValueR = 0;

while(TRUE)
    {
    // Get Left Value As Absolute Positive Value
    //------------------------------------------
    TempValue = (int)*(PSHORT)(TempDataPtr);

    TempValue = TempValue << 8;

    if(TempValue < 0)
        TempValue = 0 - TempValue;

    // Store As The Highest Peak Value If Needed
    //------------------------------------------
    if(TempValue > TempPeakValueL)
        TempPeakValueL = TempValue;

    // Get Right Value As Absolute Positive Value
    //-------------------------------------------
    TempValue = (int)*(PSHORT)(TempDataPtr + 2);

    TempValue = TempValue << 8;

    if(TempValue < 0)
        TempValue = 0 - TempValue;

    // Store As The Highest Peak Value If Needed
    //------------------------------------------
    if(TempValue > TempPeakValueR)
        TempPeakValueR = TempValue;

    TempDataPtr = TempDataPtr + 4;
    TempBuffSize = TempBuffSize - 4;

    // At End Of Buffer Store Final Results For Later Display
    //-------------------------------------------------------
    if(TempBuffSize <= 0)
        {
        // Trim Value To Max Clip Value If Needed
        //---------------------------------------
        if(TempPeakValueL > 0x007fffff)
            TempPeakValueL = 0x007fffff;

        if(TempPeakValueR > 0x007fffff)
            TempPeakValueR = 0x007fffff;

        // Scale The Peak Value To A Meter Segment Value
        //----------------------------------------------
        for(TempIndex = 0; TempIndex < MAX_METER_SEGMENTS; TempIndex++)
            {
            if(TempPeakValueL <= MeterTable[TempIndex])
                {
                if(TempIndex > 0)
                    TempIndex--;

                TempPeakValueL = TempIndex;
                break;
                }
            }

        for(TempIndex = 0; TempIndex < MAX_METER_SEGMENTS; TempIndex++)
            {
            if(TempPeakValueR <= MeterTable[TempIndex])
                {
                if(TempIndex > 0)
                    TempIndex--;

                TempPeakValueR = TempIndex;
                break;
                }
            }

        // Simulate Analog Decay For Values Smaller Than The Last Values
        //--------------------------------------------------------------
        if(TempPeakValueL < PlugInMeterLastValueL[FxProcessParamSet] - PlugInMeterDecayFactor[FxProcessParamSet])
            TempPeakValueL = PlugInMeterLastValueL[FxProcessParamSet] - PlugInMeterDecayFactor[FxProcessParamSet];
                
        if(TempPeakValueR < PlugInMeterLastValueR[FxProcessParamSet] - PlugInMeterDecayFactor[FxProcessParamSet])
            TempPeakValueR = PlugInMeterLastValueR[FxProcessParamSet] - PlugInMeterDecayFactor[FxProcessParamSet];

        // Save These Values For Next Time
        //--------------------------------
        PlugInMeterLastValueL[FxProcessParamSet] = TempPeakValueL;
        PlugInMeterLastValueR[FxProcessParamSet] = TempPeakValueR;

        // Store The Results At The Index Head Position In The Meter Array
        //----------------------------------------------------------------
        TempIndex = PlugInMeterIndexHead[FxProcessParamSet];

        PlugInMeterPos[FxProcessParamSet][TempIndex] = FxProcessPos;
        PlugInMeterValueL[FxProcessParamSet][TempIndex] = TempPeakValueL;
        PlugInMeterValueR[FxProcessParamSet][TempIndex] = TempPeakValueR;

        // Increment Index Head Counter And Set Next Array Value To End Of List Value
        //---------------------------------------------------------------------------
        TempIndex++;

        if(TempIndex >= MAX_METER_INDEXES)
            TempIndex = 0;

        PlugInMeterIndexHead[FxProcessParamSet] = TempIndex;

        PlugInMeterPos[FxProcessParamSet][TempIndex] = END_OF_LIST;
        break;
        }
    }

return;
}

//=====================
// Reset Meter Display
//=====================

void    __stdcall   ResetMeterDisplay(DWORD TempParamSet)

{
int     TempSrcXPos;
int     TempSrcYPos;
int     TempDestXPos;
int     TempDestYPos;
int     TempDestWidth;
int     TempDestHeight;
HDC     TempMemoryDC;

if(PlugInParamSetTable[TempParamSet] == 0)
    return;

if(hWndPlugIn[TempParamSet] == 0)
    return;

// Create Compatible Memory DC
//----------------------------
TempMemoryDC = CreateCompatibleDC(PlugInDC[TempParamSet]);

// Clear Lft And Rgt Meter
//------------------------
SelectObject(TempMemoryDC, hPlugInMainBitmap);

TempDestXPos = RectPlugInMeterPeakL.left;
TempDestYPos = RectPlugInMeterPeakL.top;
TempDestWidth = RectPlugInMeterR.right - RectPlugInMeterL.left + 1;
TempDestHeight = RectPlugInMeterL.bottom - RectPlugInMeterPeakL.top + 1;
    
TempSrcXPos = RectPlugInMeterPeakL.left;
TempSrcYPos = RectPlugInMeterPeakL.top;

BitBlt(PlugInDC[TempParamSet], TempDestXPos, TempDestYPos, TempDestWidth, TempDestHeight, TempMemoryDC, TempSrcXPos, TempSrcYPos, SRCCOPY);

DeleteDC(TempMemoryDC);
return;
}

//====================
// Display All Meters
//====================

void    __stdcall   DisplayAllMeters(void)

{
DWORD   TempSet;
int     TempPeakValueL;
int     TempPeakValueR;
DWORD   TempIndex;
int     TempSrcXPos;
int     TempSrcYPos;
int     TempDestXPos;
int     TempDestYPos;
int     TempDestWidth;
int     TempDestHeight;
HDC     TempMemoryDC;

// Skip If There Is Nothing To Display
//------------------------------------
if(PlugInParamSetCount == 0)
    return;

// Only Display Meters During Realtime Playback
//---------------------------------------------
if(FxProcessFlag != REALTIME_PLAYBACK)
    return;

TempMemoryDC = 0;

// Display All Active And Visible Parameter Set Meters
//----------------------------------------------------
for(TempSet = 0; TempSet < MAX_PARAM_SETS; TempSet++)
    {
    // Skip If Not Active
    //-------------------
    if(PlugInParamSetTable[TempSet] == 0)
        continue;

    // Skip If Not Visible
    //--------------------
    if(hWndPlugIn[TempSet] == 0)
        continue;

    // Skip If Bypassed
    //-----------------
    if(PlugInDataValue[TempSet][AUTO_BYPASS_SW])
        continue;

    // Find The Next Entry That Is Closest To The Current Playback Sample Position
    //----------------------------------------------------------------------------
    LoopBack:
    // Skip If The Head And Tail Index Counters Are The Same
    //------------------------------------------------------
    if(PlugInMeterIndexHead[TempSet] == PlugInMeterIndexTail[TempSet])
        continue;

    // Start Looking From The Index Tail Counter
    //------------------------------------------
    TempIndex = PlugInMeterIndexTail[TempSet];

    // Skip If Current Position Has Not Yet Reached Next Entry Position
    //-----------------------------------------------------------------
    if(FxSamplePos < PlugInMeterPos[TempSet][TempIndex])
        continue;

    // Get Values
    //-----------
    TempPeakValueL = PlugInMeterValueL[TempSet][TempIndex];
    TempPeakValueR = PlugInMeterValueR[TempSet][TempIndex];

    // Increment Index Tail Counter
    //-----------------------------
    TempIndex++;

    if(TempIndex >= MAX_METER_INDEXES)
        TempIndex = 0;

    PlugInMeterIndexTail[TempSet] = TempIndex;

    // Loopback If Current Position Is Greater Than Next Position
    //-----------------------------------------------------------
    if(FxSamplePos >= PlugInMeterPos[TempSet][TempIndex])
        goto LoopBack;

    // Create Compatible Memory DC If Needed
    //--------------------------------------
    if(TempMemoryDC == 0)
        TempMemoryDC = CreateCompatibleDC(PlugInDC[TempSet]);

    // Fill Lft Meter From Bottom To Cur Peak
    //---------------------------------------
    if(TempPeakValueL)
        {
        SelectObject(TempMemoryDC, hPlugInMeterBitmap);

        TempDestXPos = RectPlugInMeterL.left;
        TempDestYPos = RectPlugInMeterL.bottom - TempPeakValueL;
        TempDestWidth = PlugInMeterBitmapHdr.bmWidth;
        TempDestHeight = TempPeakValueL;
    
        TempSrcXPos = 0;
        TempSrcYPos = PlugInMeterBitmapHdr.bmHeight - TempPeakValueL;

        BitBlt(PlugInDC[TempSet], TempDestXPos, TempDestYPos, TempDestWidth, TempDestHeight, TempMemoryDC, TempSrcXPos, TempSrcYPos, SRCCOPY);  
        }

    // Clear Rest Of Lft Meter To Top
    //-------------------------------
    TempDestHeight = (RectPlugInMeterL.bottom - RectPlugInMeterL.top + 1) - TempPeakValueL;

    if(TempDestHeight)
        {
        SelectObject(TempMemoryDC, hPlugInMainBitmap);

        TempDestXPos = RectPlugInMeterL.left;
        TempDestYPos = RectPlugInMeterL.top;
        TempDestWidth = PlugInMeterBitmapHdr.bmWidth;
    
        TempSrcXPos = RectPlugInMeterL.left;
        TempSrcYPos = RectPlugInMeterL.top;

        BitBlt(PlugInDC[TempSet], TempDestXPos, TempDestYPos, TempDestWidth, TempDestHeight, TempMemoryDC, TempSrcXPos, TempSrcYPos, SRCCOPY);  
        }

    // Fill Lft Meter Peak If Needed
    //------------------------------
    if(TempPeakValueL >= MAX_METER_SEGMENTS - 2)
        {
        SelectObject(TempMemoryDC, hPlugInMeterBitmap);

        TempDestXPos = RectPlugInMeterPeakL.left;
        TempDestYPos = RectPlugInMeterPeakL.top;
        TempDestWidth = PlugInMeterBitmapHdr.bmWidth;
        TempDestHeight = RectPlugInMeterPeakL.bottom - RectPlugInMeterPeakL.top + 1;
    
        TempSrcXPos = 0;
        TempSrcYPos = 0;

        BitBlt(PlugInDC[TempSet], TempDestXPos, TempDestYPos, TempDestWidth, TempDestHeight, TempMemoryDC, TempSrcXPos, TempSrcYPos, SRCCOPY);  
        }

    // Fill Rgt Meter From Bottom To Cur Peak
    //---------------------------------------
    if(TempPeakValueR)
        {
        SelectObject(TempMemoryDC, hPlugInMeterBitmap);

        TempDestXPos = RectPlugInMeterR.left;
        TempDestYPos = RectPlugInMeterR.bottom - TempPeakValueR;
        TempDestWidth = PlugInMeterBitmapHdr.bmWidth;
        TempDestHeight = TempPeakValueR;
    
        TempSrcXPos = 0;
        TempSrcYPos = PlugInMeterBitmapHdr.bmHeight - TempPeakValueR;

        BitBlt(PlugInDC[TempSet], TempDestXPos, TempDestYPos, TempDestWidth, TempDestHeight, TempMemoryDC, TempSrcXPos, TempSrcYPos, SRCCOPY);  
        }

    // Clear Rest Of Lft Meter To Top
    //-------------------------------
    TempDestHeight = (RectPlugInMeterR.bottom - RectPlugInMeterR.top + 1) - TempPeakValueR;

    if(TempDestHeight)
        {
        SelectObject(TempMemoryDC, hPlugInMainBitmap);

        TempDestXPos = RectPlugInMeterR.left;
        TempDestYPos = RectPlugInMeterR.top;
        TempDestWidth = PlugInMeterBitmapHdr.bmWidth;
    
        TempSrcXPos = RectPlugInMeterR.left;
        TempSrcYPos = RectPlugInMeterR.top;

        BitBlt(PlugInDC[TempSet], TempDestXPos, TempDestYPos, TempDestWidth, TempDestHeight, TempMemoryDC, TempSrcXPos, TempSrcYPos, SRCCOPY);  
        }

    // Fill Rgt Meter Peak If Needed
    //------------------------------
    if(TempPeakValueR >= MAX_METER_SEGMENTS - 2)
        {
        SelectObject(TempMemoryDC, hPlugInMeterBitmap);

        TempDestXPos = RectPlugInMeterPeakR.left;
        TempDestYPos = RectPlugInMeterPeakR.top;
        TempDestWidth = PlugInMeterBitmapHdr.bmWidth;
        TempDestHeight = RectPlugInMeterPeakR.bottom - RectPlugInMeterPeakR.top + 1;
    
        TempSrcXPos = 0;
        TempSrcYPos = 0;

        BitBlt(PlugInDC[TempSet], TempDestXPos, TempDestYPos, TempDestWidth, TempDestHeight, TempMemoryDC, TempSrcXPos, TempSrcYPos, SRCCOPY);  
        }
    }

if(TempMemoryDC)
    DeleteDC(TempMemoryDC);

return;
}

//----
